from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.reddit.provider import RedditProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class RedditTests(OAuth2TestsMixin, TestCase):
    provider_id = RedditProvider.id

    def get_mocked_response(self):
        return [
            MockedResponse(
                HTTPStatus.OK,
                """{
        "name": "wayward710"}""",
            )
        ]

    def get_expected_to_str(self):
        return "wayward710"



import time
from http import HTTPStatus

import pytest

from allauth.account.internal.stagekit import LOGIN_SESSION_KEY
from allauth.account.models import EmailAddress
from allauth.account.stages import LoginByCodeStage
from allauth.headless.constants import Flow


@pytest.fixture
def get_last_login_code(mailoutbox):
    def f():
        return [line for line in mailoutbox[-1].body.splitlines() if len(line) == 9][0]

    return f


def test_login_by_code(headless_reverse, user, client, mailoutbox, get_last_login_code):
    resp = client.post(
        headless_reverse("headless:account:request_login_code"),
        data={"email": user.email},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    data = resp.json()
    assert [f for f in data["data"]["flows"] if f["id"] == Flow.LOGIN_BY_CODE][0][
        "is_pending"
    ]
    assert len(mailoutbox) == 1
    code = get_last_login_code()
    resp = client.post(
        headless_reverse("headless:account:confirm_login_code"),
        data={"code": code},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.OK
    data = resp.json()
    assert data["meta"]["is_authenticated"]


def test_login_by_code_rate_limit(
    headless_reverse, user, client, mailoutbox, settings, enable_cache
):
    settings.ACCOUNT_RATE_LIMITS = {"request_login_code": "1/m/ip"}
    for attempt in range(2):
        resp = client.post(
            headless_reverse("headless:account:request_login_code"),
            data={"email": user.email},
            content_type="application/json",
        )
        expected_code = HTTPStatus.BAD_REQUEST if attempt else HTTPStatus.UNAUTHORIZED
        assert resp.status_code == expected_code
        data = resp.json()
        assert data["status"] == expected_code
        if expected_code == HTTPStatus.BAD_REQUEST:
            assert data["errors"] == [
                {
                    "code": "too_many_login_attempts",
                    "message": "Too many failed login attempts. Try again later.",
                    "param": "email",
                },
            ]


def test_login_by_code_max_attemps(headless_reverse, user, client, settings):
    settings.ACCOUNT_LOGIN_BY_CODE_MAX_ATTEMPTS = 2
    resp = client.post(
        headless_reverse("headless:account:request_login_code"),
        data={"email": user.email},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    for i in range(3):
        resp = client.post(
            headless_reverse("headless:account:confirm_login_code"),
            data={"code": "wrong"},
            content_type="application/json",
        )
        session_resp = client.get(
            headless_reverse("headless:account:current_session"),
            data={"code": "wrong"},
            content_type="application/json",
        )
        assert session_resp.status_code == HTTPStatus.UNAUTHORIZED
        pending_flows = [
            f for f in session_resp.json()["data"]["flows"] if f.get("is_pending")
        ]
        if i >= 1:
            assert (
                resp.status_code == HTTPStatus.CONFLICT
                if i >= 2
                else HTTPStatus.BAD_REQUEST
            )
            assert len(pending_flows) == 0
        else:
            assert resp.status_code == HTTPStatus.BAD_REQUEST
            assert len(pending_flows) == 1


def test_login_by_code_required(
    client, settings, user_factory, password_factory, headless_reverse, mailoutbox
):
    settings.ACCOUNT_LOGIN_BY_CODE_REQUIRED = True
    password = password_factory()
    user = user_factory(password=password, email_verified=False)
    email_address = EmailAddress.objects.get(email=user.email)
    assert not email_address.verified
    resp = client.post(
        headless_reverse("headless:account:login"),
        data={
            "username": user.username,
            "password": password,
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    pending_flow = [f for f in resp.json()["data"]["flows"] if f.get("is_pending")][0][
        "id"
    ]
    assert pending_flow == Flow.LOGIN_BY_CODE
    code = [line for line in mailoutbox[0].body.splitlines() if len(line) == 9][0]
    resp = client.post(
        headless_reverse("headless:account:confirm_login_code"),
        data={"code": code},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.OK
    data = resp.json()
    assert data["meta"]["is_authenticated"]
    email_address.refresh_from_db()
    assert email_address.verified


def test_login_by_code_expired(headless_reverse, user, client, mailoutbox):
    resp = client.post(
        headless_reverse("headless:account:request_login_code"),
        data={"email": user.email},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    data = resp.json()
    assert [f for f in data["data"]["flows"] if f["id"] == Flow.LOGIN_BY_CODE][0][
        "is_pending"
    ]
    assert len(mailoutbox) == 1
    code = [line for line in mailoutbox[0].body.splitlines() if len(line) == 9][0]

    # Expire code
    session = client.headless_session()
    login = session[LOGIN_SESSION_KEY]
    login["state"]["stages"][LoginByCodeStage.key]["data"]["at"] = (
        time.time() - 24 * 60 * 60
    )
    session["account_login"] = login
    session.save()

    # Post valid code
    resp = client.post(
        headless_reverse("headless:account:confirm_login_code"),
        data={"code": code},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.CONFLICT


def test_post_login_code_when_flow_not_pending(
    db,
    client,
    settings_impacting_urls,
    headless_reverse,
    mailoutbox,
    get_last_email_verification_code,
):
    with settings_impacting_urls(
        ACCOUNT_LOGIN_METHODS={"email"},
        ACCOUNT_SIGNUP_FIELDS=["email*"],
        ACCOUNT_EMAIL_VERIFICATION="mandatory",
        ACCOUNT_LOGIN_BY_CODE_ENABLED=True,
        ACCOUNT_LOGIN_BY_CODE_REQUIRED=False,
        ACCOUNT_EMAIL_VERIFICATION_BY_CODE_ENABLED=True,
    ):
        resp = client.post(
            headless_reverse("headless:account:signup"),
            data={"email": "user@email.org"},
            content_type="application/json",
        )
        assert resp.status_code == HTTPStatus.UNAUTHORIZED
        resp = client.post(
            headless_reverse("headless:account:confirm_login_code"),
            data={"code": "123"},
            content_type="application/json",
        )
        assert resp.status_code == HTTPStatus.CONFLICT


def test_login_by_code_resend_limited(
    settings, headless_reverse, user, client, mailoutbox, get_last_login_code
):
    settings.ACCOUNT_LOGIN_BY_CODE_SUPPORTS_RESEND = 2
    resp = client.post(
        headless_reverse("headless:account:request_login_code"),
        data={"email": user.email},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    code = get_last_login_code()
    for i in range(3):
        resp = client.post(
            headless_reverse("headless:account:resend_login_code"),
        )
        assert resp.status_code == (HTTPStatus.CONFLICT if i == 2 else HTTPStatus.OK)
        new_code = get_last_login_code()
        if i == 2:
            assert new_code == code
        else:
            assert new_code != code
            code = new_code



from dataclasses import dataclass, field
from typing import Optional

from allauth.headless.spec.internal.openapikit import spec_for_dataclass


@dataclass
class NestedDataClass:
    string: str
    integer: int = field(
        metadata={
            "description": "Some nested int",
            "example": 42,
        }
    )


@dataclass
class ExampleDataclass:
    optional_integer: Optional[int]
    integer: int
    optional_string: Optional[str]
    string: str
    number: float = field(
        metadata={
            "description": "Some float",
            "example": "3.14",
        }
    )
    nested: Optional[NestedDataClass]


def test_spec_for_dataclass():
    spec = spec_for_dataclass(ExampleDataclass)
    assert spec == (
        {
            "properties": {
                "integer": {
                    "type": "integer",
                },
                "number": {
                    "description": "Some float",
                    "example": "3.14",
                    "format": "float",
                    "type": "number",
                },
                "optional_integer": {
                    "type": "integer",
                },
                "optional_string": {
                    "type": "string",
                },
                "string": {
                    "type": "string",
                },
                "nested": {
                    "example": {"integer": 42},
                    "properties": {
                        "string": {
                            "type": "string",
                        },
                        "integer": {
                            "description": "Some nested int",
                            "example": 42,
                            "type": "integer",
                        },
                    },
                    "required": [
                        "string",
                        "integer",
                    ],
                    "type": "object",
                },
            },
            "required": [
                "integer",
                "string",
                "number",
            ],
            "type": "object",
        },
        {
            "number": "3.14",
        },
    )



from http import HTTPStatus

from allauth.usersessions.models import UserSession


def test_flow(client, user, user_password, headless_reverse, settings):
    settings.ACCOUNT_LOGIN_METHODS = {"email"}
    resp = client.post(
        headless_reverse("headless:account:login"),
        data={
            "email": user.email,
            "password": user_password,
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.OK
    resp = client.get(headless_reverse("headless:usersessions:sessions"))
    assert resp.status_code == HTTPStatus.OK
    data = resp.json()
    assert len(data["data"]) == 1
    session_pk = data["data"][0]["id"]
    assert UserSession.objects.filter(pk=session_pk).exists()
    resp = client.delete(
        headless_reverse("headless:usersessions:sessions"),
        data={"sessions": [session_pk]},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    assert not UserSession.objects.filter(pk=session_pk).exists()



from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class FoursquareOAuth2Adapter(OAuth2Adapter):
    provider_id = "foursquare"
    access_token_url = "https://foursquare.com/oauth2/access_token"  # nosec
    # Issue ?? -- this one authenticates over and over again...
    # authorize_url = 'https://foursquare.com/oauth2/authorize'
    authorize_url = "https://foursquare.com/oauth2/authenticate"
    profile_url = "https://api.foursquare.com/v2/users/self"

    def complete_login(self, request, app, token, **kwargs):
        # Foursquare needs a version number for their API requests as
        # documented here
        # https://developer.foursquare.com/overview/versioning
        with get_adapter().get_requests_session() as sess:
            params = {"oauth_token": token.token, "v": "20140116"}
            resp = sess.get(self.profile_url, params=params)
            extra_data = resp.json()["response"]["user"]
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(FoursquareOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(FoursquareOAuth2Adapter)



from http import HTTPStatus

from django.test import RequestFactory, TestCase
from django.utils.http import base36_to_int, int_to_base36
from django.views import csrf

from allauth import app_settings, utils


class BasicTests(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_generate_unique_username(self):
        examples = [
            ("a.b-c@example.com", "a.b-c"),
            ("Üsêrnamê", "username"),
            ("User Name", "user_name"),
            ("", "user"),
        ]
        for input, username in examples:
            self.assertEqual(utils.generate_unique_username([input]), username)

    def test_build_absolute_uri(self):
        request = None
        if not app_settings.SITES_ENABLED:
            request = self.factory.get("/")
            request.META["SERVER_NAME"] = "example.com"
        self.assertEqual(
            utils.build_absolute_uri(request, "/foo"), "http://example.com/foo"
        )
        self.assertEqual(
            utils.build_absolute_uri(request, "/foo", protocol="ftp"),
            "ftp://example.com/foo",
        )
        self.assertEqual(
            utils.build_absolute_uri(request, "http://foo.com/bar"),
            "http://foo.com/bar",
        )

    def test_int_to_base36(self):
        n = 55798679658823689999
        b36 = "brxk553wvxbf3"
        assert int_to_base36(n) == b36
        assert base36_to_int(b36) == n

    def test_templatetag_with_csrf_failure(self):
        # Generate a fictitious GET request
        if not app_settings.SOCIALACCOUNT_ENABLED:
            return

        from allauth.socialaccount.models import SocialApp

        app = SocialApp.objects.create(provider="google")
        if app_settings.SITES_ENABLED:
            from django.contrib.sites.models import Site

            app.sites.add(Site.objects.get_current())

        request = self.factory.get("/tests/test_403_csrf.html")
        # Simulate a CSRF failure by calling the View directly
        # This template is using the `provider_login_url` templatetag
        response = csrf.csrf_failure(request, template_name="test_403_csrf.html")
        # Ensure that CSRF failures with this template
        # tag succeed with the expected 403 response
        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)



from http import HTTPStatus

import pytest

from allauth.headless.constants import Client, Flow


@pytest.mark.parametrize("trust", [False, True])
def test_auth_unverified_email_and_mfa(
    client,
    user_with_totp,
    user_password,
    settings_impacting_urls,
    totp_validation_bypass,
    headless_reverse,
    headless_client,
    trust,
):
    with settings_impacting_urls(
        ACCOUNT_LOGIN_METHODS={"email"},
        ACCOUNT_EMAIL_VERIFICATION="mandatory",
        ACCOUNT_LOGIN_ON_EMAIL_CONFIRMATION=True,
        MFA_TRUST_ENABLED=True,
    ):
        # Login
        resp = client.post(
            headless_reverse("headless:account:login"),
            data={
                "email": user_with_totp.email,
                "password": user_password,
            },
            content_type="application/json",
        )

        # TOTP is the next stage
        assert resp.status_code == HTTPStatus.UNAUTHORIZED
        data = resp.json()
        assert [f for f in data["data"]["flows"] if f["id"] == Flow.MFA_AUTHENTICATE][
            0
        ]["is_pending"]
        with totp_validation_bypass():
            resp = client.post(
                headless_reverse("headless:mfa:authenticate"),
                data={"code": "bad"},
                content_type="application/json",
            )

        if headless_client == Client.APP:
            # App client does not support trust
            assert resp.status_code == HTTPStatus.OK
        else:
            # Trust stage is pending
            assert resp.status_code == HTTPStatus.UNAUTHORIZED
            data = resp.json()
            assert [f for f in data["data"]["flows"] if f["id"] == Flow.MFA_TRUST][0][
                "is_pending"
            ]

            # Indicate trust Y/N
            resp = client.post(
                headless_reverse("headless:mfa:trust"),
                data={"trust": trust},
                content_type="application/json",
            )

            # Logout
            assert resp.status_code == HTTPStatus.OK
            resp = client.delete(
                headless_reverse("headless:account:current_session"),
            )
            assert resp.status_code == HTTPStatus.UNAUTHORIZED

            # Login
            resp = client.post(
                headless_reverse("headless:account:login"),
                data={
                    "email": user_with_totp.email,
                    "password": user_password,
                },
                content_type="application/json",
            )

            # Trust used?
            assert resp.status_code == (
                HTTPStatus.OK if trust else HTTPStatus.UNAUTHORIZED
            )



from unittest.mock import patch

from django.contrib.auth import get_user_model
from django.test import TestCase
from django.test.utils import override_settings

import pytest

from allauth.account import app_settings
from allauth.account.auth_backends import AuthenticationBackend


class AuthenticationBackendTests(TestCase):
    def setUp(self):
        user = get_user_model().objects.create(
            is_active=True, email="john@example.com", username="john"
        )
        user.set_password(user.username)
        user.save()
        self.user = user

    @override_settings(
        ACCOUNT_LOGIN_METHODS={app_settings.LoginMethod.USERNAME}
    )  # noqa
    def test_auth_by_username(self):
        user = self.user
        backend = AuthenticationBackend()
        self.assertEqual(
            backend.authenticate(
                request=None, username=user.username, password=user.username
            ).pk,
            user.pk,
        )
        self.assertEqual(
            backend.authenticate(
                request=None, username=user.email, password=user.username
            ),
            None,
        )

    @override_settings(ACCOUNT_LOGIN_METHODS={app_settings.LoginMethod.EMAIL})  # noqa
    def test_auth_by_email(self):
        user = self.user
        backend = AuthenticationBackend()
        self.assertEqual(
            backend.authenticate(
                request=None, username=user.email, password=user.username
            ).pk,
            user.pk,
        )
        self.assertEqual(
            backend.authenticate(
                request=None, username=user.username, password=user.username
            ),
            None,
        )

    @override_settings(
        ACCOUNT_LOGIN_METHODS={
            app_settings.LoginMethod.EMAIL,
            app_settings.LoginMethod.USERNAME,
        }
    )  # noqa
    def test_auth_by_username_or_email(self):
        user = self.user
        backend = AuthenticationBackend()
        self.assertEqual(
            backend.authenticate(
                request=None, username=user.email, password=user.username
            ).pk,
            user.pk,
        )
        self.assertEqual(
            backend.authenticate(
                request=None, username=user.username, password=user.username
            ).pk,
            user.pk,
        )


@pytest.mark.parametrize(
    "login_methods",
    [
        {app_settings.LoginMethod.EMAIL},
        {app_settings.LoginMethod.USERNAME},
        {app_settings.LoginMethod.USERNAME, app_settings.LoginMethod.EMAIL},
    ],
)
def test_account_enumeration_timing_attack(user, db, rf, settings, login_methods):
    settings.ACCOUNT_LOGIN_METHODS = login_methods
    with patch("django.contrib.auth.models.User.set_password") as set_password_mock:
        with patch(
            "django.contrib.auth.models.User.check_password", new=set_password_mock
        ):
            backend = AuthenticationBackend()
            backend.authenticate(
                rf.get("/"),
                email="not@known.org",
                username="not-known",
                password="secret",
            )
            set_password_mock.assert_called_once()
            set_password_mock.reset_mock()
            backend.authenticate(rf.get("/"), username=user.username, password="secret")
            set_password_mock.assert_called_once()
            set_password_mock.reset_mock()
            backend.authenticate(
                rf.get("/"), email=user.email, username="not-known", password="secret"
            )
            set_password_mock.assert_called_once()



import time
from http import HTTPStatus
from unittest.mock import ANY, patch

from django.conf import settings
from django.core.cache import cache
from django.test import Client
from django.urls import reverse

import pytest
from pytest_django.asserts import assertTemplateUsed

from allauth.account import app_settings
from allauth.account.authentication import AUTHENTICATION_METHODS_SESSION_KEY
from allauth.mfa.adapter import get_adapter
from allauth.mfa.models import Authenticator


def test_activate_totp_with_incorrect_code(auth_client, reauthentication_bypass):
    with reauthentication_bypass():
        resp = auth_client.get(reverse("mfa_activate_totp"))
        resp = auth_client.post(
            reverse("mfa_activate_totp"),
            {
                "code": "123",
            },
        )
    assert resp.context["form"].errors == {
        "code": [get_adapter().error_messages["incorrect_code"]]
    }


@pytest.mark.parametrize("email_verified", [False])
@pytest.mark.parametrize("method", ["get", "post"])
def test_activate_totp_with_unverified_email(
    auth_client, user, totp_validation_bypass, reauthentication_bypass, method
):
    with reauthentication_bypass():
        if method == "get":
            resp = auth_client.get(reverse("mfa_activate_totp"))
        else:
            resp = auth_client.post(reverse("mfa_activate_totp"), {"code": "123"})
    assert resp["location"] == reverse("mfa_index")


def test_activate_totp_success(
    auth_client,
    totp_validation_bypass,
    user,
    reauthentication_bypass,
    settings,
    mailoutbox,
):
    settings.ACCOUNT_EMAIL_NOTIFICATIONS = True
    with reauthentication_bypass():
        resp = auth_client.get(reverse("mfa_activate_totp"))
        with totp_validation_bypass():
            resp = auth_client.post(
                reverse("mfa_activate_totp"),
                {
                    "code": "123",
                },
            )
    assert resp["location"] == reverse("mfa_view_recovery_codes")
    assert Authenticator.objects.filter(
        user=user, type=Authenticator.Type.TOTP
    ).exists()
    assert Authenticator.objects.filter(
        user=user, type=Authenticator.Type.RECOVERY_CODES
    ).exists()
    assert len(mailoutbox) == 1
    assert "Authenticator App Activated" in mailoutbox[0].subject
    assert "Authenticator app activated." in mailoutbox[0].body


def test_deactivate_totp_success(
    auth_client, user_with_totp, user_password, settings, mailoutbox
):
    settings.ACCOUNT_EMAIL_NOTIFICATIONS = True
    resp = auth_client.get(reverse("mfa_deactivate_totp"))
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"].startswith(reverse("account_reauthenticate"))
    resp = auth_client.post(resp["location"], {"password": user_password})
    assert resp.status_code == HTTPStatus.FOUND
    resp = auth_client.post(reverse("mfa_deactivate_totp"))
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("mfa_index")
    assert len(mailoutbox) == 1
    assert "Authenticator App Deactivated" in mailoutbox[0].subject
    assert "Authenticator app deactivated." in mailoutbox[0].body


def test_user_without_totp_deactivate_totp(auth_client):
    resp = auth_client.get(reverse("mfa_deactivate_totp"))
    assert resp.status_code == HTTPStatus.NOT_FOUND


def test_user_with_totp_activate_totp(
    auth_client, user_with_totp, reauthentication_bypass
):
    with reauthentication_bypass():
        resp = auth_client.get(reverse("mfa_activate_totp"))
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("mfa_deactivate_totp")


def test_totp_login(client, user_with_totp, user_password, totp_validation_bypass):
    resp = client.post(
        reverse("account_login"),
        {"login": user_with_totp.username, "password": user_password},
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("mfa_authenticate")
    resp = client.get(reverse("mfa_authenticate"))
    assert resp.context["request"].user.is_anonymous
    resp = client.post(reverse("mfa_authenticate"), {"code": "123"})
    assert resp.context["form"].errors == {
        "code": [get_adapter().error_messages["incorrect_code"]]
    }
    with totp_validation_bypass():
        resp = client.post(
            reverse("mfa_authenticate"),
            {"code": "123"},
        )
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == settings.LOGIN_REDIRECT_URL
    assert client.session[AUTHENTICATION_METHODS_SESSION_KEY] == [
        {"method": "password", "at": ANY, "username": user_with_totp.username},
        {"method": "mfa", "at": ANY, "id": ANY, "type": Authenticator.Type.TOTP},
    ]


def test_totp_login_rate_limit(
    settings, enable_cache, user_with_totp, user_password, client
):
    settings.ACCOUNT_LOGIN_ATTEMPTS_LIMIT = 3
    resp = client.post(
        reverse("account_login"),
        {"login": user_with_totp.username, "password": user_password},
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("mfa_authenticate")
    for i in range(5):
        is_locked = i >= 3
        resp = client.post(
            reverse("mfa_authenticate"),
            {
                "code": "wrong",
            },
        )
        assert resp.context["form"].errors == {
            "code": [
                (
                    "Too many failed login attempts. Try again later."
                    if is_locked
                    else "Incorrect code."
                )
            ]
        }


def test_cannot_deactivate_totp(auth_client, user_with_totp, user_password):
    with patch(
        "allauth.mfa.adapter.DefaultMFAAdapter.can_delete_authenticator"
    ) as cda_mock:
        cda_mock.return_value = False
        resp = auth_client.get(reverse("mfa_deactivate_totp"))
        assert resp.status_code == HTTPStatus.FOUND
        assert resp["location"].startswith(reverse("account_reauthenticate"))
        resp = auth_client.post(resp["location"], {"password": user_password})
        assert resp.status_code == HTTPStatus.FOUND
        resp = auth_client.get(reverse("mfa_deactivate_totp"))
        # When we GET, the form validation error is already on screen
        assert resp.context["form"].errors == {
            "__all__": [get_adapter().error_messages["cannot_delete_authenticator"]],
        }
        # And, when we POST anyway, it does not work
        resp = auth_client.post(reverse("mfa_deactivate_totp"))
        assert resp.status_code == HTTPStatus.OK
        assert resp.context["form"].errors == {
            "__all__": [get_adapter().error_messages["cannot_delete_authenticator"]],
        }


def test_totp_code_reuse(
    user_with_totp, user_password, totp_validation_bypass, enable_cache
):
    for code, time_lapse, expect_success in [
        # First use of code, SUCCESS
        ("123", False, True),
        # Second use, no time elapsed: FAIL
        ("123", False, False),
        # Different code, no time elapsed: SUCCESS
        ("456", False, True),
        # Again, previous code, no time elapsed: FAIL
        ("123", False, False),
        # Previous code, but time elapsed: SUCCESS
        ("123", True, True),
    ]:
        if time_lapse:
            cache.clear()
        client = Client()
        resp = client.post(
            reverse("account_login"),
            {"login": user_with_totp.username, "password": user_password},
        )
        assert resp.status_code == HTTPStatus.FOUND
        assert resp["location"] == reverse("mfa_authenticate")
        # Note that this bypass only bypasses the actual code check, not the
        # re-use check we're testing here.
        with totp_validation_bypass():
            resp = client.post(
                reverse("mfa_authenticate"),
                {"code": code},
            )
        if expect_success:
            assert resp.status_code == HTTPStatus.FOUND
            assert resp["location"] == settings.LOGIN_REDIRECT_URL
        else:
            assert resp.status_code == HTTPStatus.OK
            assert resp.context["form"].errors == {
                "code": [get_adapter().error_messages["incorrect_code"]]
            }


def test_totp_stage_expires(client, user_with_totp, user_password):
    resp = client.post(
        reverse("account_login"),
        {"login": user_with_totp.username, "password": user_password},
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("mfa_authenticate")
    resp = client.get(reverse("mfa_authenticate"))
    assert resp.status_code == HTTPStatus.OK
    assertTemplateUsed(resp, "mfa/authenticate.html")
    with patch(
        "allauth.account.internal.stagekit.time.time",
        return_value=time.time() + 1.1 * app_settings.LOGIN_TIMEOUT,
    ):
        resp = client.get(reverse("mfa_authenticate"))
        assert resp.status_code == HTTPStatus.FOUND
        assert resp["location"] == reverse("account_login")



from rest_framework.response import Response
from rest_framework.views import APIView

from allauth.headless.contrib.rest_framework.authentication import (
    JWTTokenAuthentication,
)


class ResourceView(APIView):
    authentication_classes = [JWTTokenAuthentication]

    def get(self, request, *args, **kwargs):
        data = {"resource": "ok"}
        if "userinfo" in request.GET:
            data["user_email"] = request.user.email
        return Response(data)



import pytest

from allauth.headless.socialaccount.inputs import ProviderTokenInput


@pytest.mark.parametrize("client_id", ["client1", "client2"])
def test_provider_token_multiple_apps(settings, db, client_id):
    gsettings = {
        "APPS": [
            {"client_id": "client1", "secret": "secret"},
            {"client_id": "client2", "secret": "secret"},
        ]
    }
    settings.SOCIALACCOUNT_PROVIDERS = {"google": gsettings}

    inp = ProviderTokenInput(
        {
            "provider": "google",
            "process": "login",
            "token": {"client_id": client_id, "id_token": "it", "access_token": "at"},
        }
    )
    assert not inp.is_valid()
    assert inp.cleaned_data["provider"].app.client_id == client_id
    assert inp.errors == {"token": ["Invalid token."]}


def test_provider_token_client_id_required(settings, db):
    inp = ProviderTokenInput(
        {
            "provider": "google",
            "process": "login",
            "token": {"id_token": "it", "access_token": "at"},
        }
    )
    assert not inp.is_valid()
    assert inp.errors == {"token": ["`client_id` required."]}



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.instagram.provider import InstagramProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class InstagramTests(OAuth2TestsMixin, TestCase):
    provider_id = InstagramProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """
        {
            "username": "georgewhewell",
            "bio": "",
            "website": "",
            "profile_picture":
            "http://images.ak.instagram.com/profiles/profile_11428116_75sq_1339547159.jpg",
            "full_name": "georgewhewell",
            "counts": {
              "media": 74,
              "followed_by": 91,
              "follows": 104
            },
            "id": "11428116"
        }""",
        )  # noqa

    def get_expected_to_str(self):
        return "georgewhewell"



import copy
from http import HTTPStatus
from unittest.mock import ANY

import pytest


@pytest.mark.parametrize(
    "has_password,request_data,response_data,status_code",
    [
        # Wrong current password
        (
            True,
            {"current_password": "wrong", "new_password": "{password_factory}"},
            {
                "status": HTTPStatus.BAD_REQUEST,
                "errors": [
                    {
                        "param": "current_password",
                        "message": "Please type your current password.",
                        "code": "enter_current_password",
                    }
                ],
            },
            HTTPStatus.BAD_REQUEST,
        ),
        # Happy flow, regular password change
        (
            True,
            {
                "current_password": "{user_password}",
                "new_password": "{password_factory}",
            },
            {
                "status": HTTPStatus.OK,
                "meta": {"is_authenticated": True},
                "data": {
                    "user": ANY,
                    "methods": [],
                },
            },
            HTTPStatus.OK,
        ),
        # New password does not match constraints
        (
            True,
            {
                "current_password": "{user_password}",
                "new_password": "a",
            },
            {
                "status": HTTPStatus.BAD_REQUEST,
                "errors": [
                    {
                        "param": "new_password",
                        "code": "password_too_short",
                        "message": "This password is too short. It must contain at least 6 characters.",
                    }
                ],
            },
            HTTPStatus.BAD_REQUEST,
        ),
        # New password not empty
        (
            True,
            {
                "current_password": "{user_password}",
                "new_password": "",
            },
            {
                "status": HTTPStatus.BAD_REQUEST,
                "errors": [
                    {
                        "param": "new_password",
                        "code": "required",
                        "message": "This field is required.",
                    }
                ],
            },
            HTTPStatus.BAD_REQUEST,
        ),
        # Current password not blank
        (
            True,
            {
                "current_password": "",
                "new_password": "{password_factory}",
            },
            {
                "status": HTTPStatus.BAD_REQUEST,
                "errors": [
                    {
                        "param": "current_password",
                        "message": "This field is required.",
                        "code": "required",
                    }
                ],
            },
            HTTPStatus.BAD_REQUEST,
        ),
        # Current password missing
        (
            True,
            {
                "new_password": "{password_factory}",
            },
            {
                "status": HTTPStatus.BAD_REQUEST,
                "errors": [
                    {
                        "param": "current_password",
                        "message": "This field is required.",
                        "code": "required",
                    }
                ],
            },
            HTTPStatus.BAD_REQUEST,
        ),
        # Current password not set, happy flow
        (
            False,
            {
                "current_password": "",
                "new_password": "{password_factory}",
            },
            {
                "status": HTTPStatus.OK,
                "meta": {"is_authenticated": True},
                "data": {
                    "user": ANY,
                    "methods": [],
                },
            },
            HTTPStatus.OK,
        ),
        # Current password not set, current_password absent
        (
            False,
            {
                "new_password": "{password_factory}",
            },
            {
                "status": HTTPStatus.OK,
                "meta": {"is_authenticated": True},
                "data": {
                    "user": ANY,
                    "methods": [],
                },
            },
            HTTPStatus.OK,
        ),
    ],
)
def test_change_password(
    auth_client,
    user,
    request_data,
    response_data,
    status_code,
    has_password,
    user_password,
    password_factory,
    settings,
    mailoutbox,
    headless_reverse,
    headless_client,
):
    request_data = copy.deepcopy(request_data)
    response_data = copy.deepcopy(response_data)
    settings.ACCOUNT_EMAIL_NOTIFICATIONS = True
    if not has_password:
        user.set_unusable_password()
        user.save(update_fields=["password"])
        auth_client.force_login(user)
    if request_data.get("current_password") == "{user_password}":
        request_data["current_password"] = user_password
    if request_data.get("new_password") == "{password_factory}":
        request_data["new_password"] = password_factory()
    resp = auth_client.post(
        headless_reverse("headless:account:change_password"),
        data=request_data,
        content_type="application/json",
    )
    assert resp.status_code == status_code
    resp_json = resp.json()
    if headless_client == "app" and resp.status_code == HTTPStatus.OK:
        response_data["meta"]["session_token"] = ANY
    assert resp_json == response_data
    user.refresh_from_db()
    if resp.status_code == HTTPStatus.OK:
        assert user.check_password(request_data["new_password"])
        assert len(mailoutbox) == 1
    else:
        assert user.check_password(user_password)
        assert len(mailoutbox) == 0


def test_change_password_rate_limit(
    enable_cache,
    auth_client,
    user,
    user_password,
    password_factory,
    settings,
    headless_reverse,
):
    settings.ACCOUNT_RATE_LIMITS = {"change_password": "1/m/ip"}
    for attempt in range(2):
        new_password = password_factory()
        resp = auth_client.post(
            headless_reverse("headless:account:change_password"),
            data={
                "current_password": user_password,
                "new_password": new_password,
            },
            content_type="application/json",
        )
        user_password = new_password
        expected_status = (
            HTTPStatus.OK if attempt == 0 else HTTPStatus.TOO_MANY_REQUESTS
        )
        assert resp.status_code == expected_status
        assert resp.json()["status"] == expected_status



import pytest

from allauth.idp.oidc.internal.oauthlib.utils import get_uri


@pytest.mark.parametrize(
    "input,output",
    [
        ("/foo?q=param", "/foo?q=param"),
        (
            "/foo?q=a|b&c=c^d&p=`",
            "/foo?q=a%7Cb&c=c%5Ed&p=%60",
        ),
    ],
)
def test_get_uri(rf, input, output):
    request = rf.get(input)
    assert get_uri(request) == output



from unittest.mock import Mock, patch

from django.conf import settings
from django.test import TestCase
from django.urls import reverse

from openid.yadis.discover import DiscoveryFailure
from pytest_django.asserts import assertRedirects

from allauth.socialaccount.providers.steam import views
from allauth.socialaccount.providers.steam.provider import SteamOpenIDProvider
from tests.apps.socialaccount.base import setup_app


class SteamTests(TestCase):
    def setUp(self):
        self.app = setup_app(SteamOpenIDProvider.id)

    def test_redirect(self):
        with patch(
            "allauth.socialaccount.providers.steam.provider._openid_consumer"
        ) as consumer_mock:
            consumer_client = Mock()
            begin = Mock()
            auth_request = Mock()
            redirectURL = Mock()
            consumer_mock.return_value = consumer_client
            consumer_client.begin = begin
            begin.return_value = auth_request
            auth_request.redirectURL = redirectURL
            redirectURL.return_value = "https://steamcommunity.com/openid/login?XXX"

            resp = self.client.post(reverse(views.steam_login))
            assertRedirects(
                resp,
                "https://steamcommunity.com/openid/login?XXX",
                fetch_redirect_response=False,
            )

    def test_redirect_error(self):
        with patch(
            "allauth.socialaccount.providers.steam.provider._openid_consumer"
        ) as consumer_mock:
            consumer_client = Mock()
            begin = Mock()
            consumer_mock.return_value = consumer_client
            consumer_client.begin = begin

            def discovery_failure(*args, **kwargs):
                raise DiscoveryFailure(
                    "HTTP Response status from identity URL host is not 200. Got status 502",
                    None,
                )

            begin.side_effect = discovery_failure

            resp = self.client.post(reverse(views.steam_login))
            template_ext = getattr(settings, "ACCOUNT_TEMPLATE_EXTENSION", "html")
            self.assertTemplateUsed(
                resp, f"socialaccount/authentication_error.{template_ext}"
            )



from http import HTTPStatus


def test_resource(db, client, access_token_generator, user, oidc_client):
    token, _ = access_token_generator(
        client=oidc_client, user=user, scopes=["view-resource"]
    )
    resp = client.get("/idp/ninja/resource", HTTP_AUTHORIZATION=f"bearer {token}")
    assert resp.status_code == HTTPStatus.OK
    assert resp.json()["user_email"] == user.email


def test_resource_using_id_token(db, client, id_token_generator, user, oidc_client):
    token = id_token_generator(client=oidc_client, user=user)
    resp = client.get("/idp/ninja/resource", HTTP_AUTHORIZATION=f"bearer {token}")
    assert resp.status_code == HTTPStatus.UNAUTHORIZED


def test_resource_forbidden(db, client, access_token_generator, user, oidc_client):
    token, _ = access_token_generator(
        client=oidc_client, user=user, scopes=["other-resource"]
    )
    resp = client.get("/idp/ninja/resource", HTTP_AUTHORIZATION=f"bearer {token}")
    assert resp.status_code == HTTPStatus.UNAUTHORIZED


def test_resource_user_inactive(db, client, access_token_generator, user, oidc_client):
    user.is_active = False
    user.save(update_fields=["is_active"])
    token, _ = access_token_generator(
        client=oidc_client, user=user, scopes=["view-resource"]
    )
    resp = client.get("/idp/ninja/resource", HTTP_AUTHORIZATION=f"bearer {token}")
    assert resp.status_code == HTTPStatus.UNAUTHORIZED



from ninja import NinjaAPI

from allauth.headless.contrib.ninja.security import jwt_token_auth


api = NinjaAPI(urls_namespace="headless")


@api.get("/resource", auth=[jwt_token_auth])
def resource(request):
    data = {"resource": "ok"}
    if "userinfo" in request.GET:
        data["user_email"] = request.user.email
    return data



#!/usr/bin/env python
import os


os.environ.setdefault("DJANGO_SETTINGS_MODULE", "tests.projects.regular.settings")
from django.core import management  # noqa: E402


if __name__ == "__main__":
    management.execute_from_command_line()



from http import HTTPStatus
from unittest.mock import ANY

from django.conf import settings
from django.urls import reverse

from allauth.account.authentication import AUTHENTICATION_METHODS_SESSION_KEY
from allauth.mfa import app_settings
from allauth.mfa.adapter import get_adapter
from allauth.mfa.models import Authenticator


def test_generate_recovery_codes_require_other_authenticator(
    auth_client, user, settings, reauthentication_bypass
):
    with reauthentication_bypass():
        resp = auth_client.post(reverse("mfa_generate_recovery_codes"))
    assert resp.context["form"].errors == {
        "__all__": [
            "You cannot generate recovery codes without having two-factor authentication enabled."
        ]
    }
    assert not Authenticator.objects.filter(user=user).exists()


def test_download_recovery_codes(auth_client, user_with_recovery_codes, user_password):
    resp = auth_client.get(reverse("mfa_download_recovery_codes"))
    assert resp["location"].startswith(reverse("account_reauthenticate"))
    resp = auth_client.post(resp["location"], {"password": user_password})
    assert resp.status_code == HTTPStatus.FOUND
    resp = auth_client.get(resp["location"])
    assert resp["content-disposition"] == 'attachment; filename="recovery-codes.txt"'


def test_view_recovery_codes(auth_client, user_with_recovery_codes, user_password):
    resp = auth_client.get(reverse("mfa_view_recovery_codes"))
    assert resp["location"].startswith(reverse("account_reauthenticate"))
    resp = auth_client.post(resp["location"], {"password": user_password})
    assert resp.status_code == HTTPStatus.FOUND
    resp = auth_client.get(resp["location"])
    assert len(resp.context["unused_codes"]) == app_settings.RECOVERY_CODE_COUNT


def test_generate_recovery_codes(
    auth_client, user_with_recovery_codes, user_password, settings, mailoutbox
):
    settings.ACCOUNT_EMAIL_NOTIFICATIONS = True
    rc = Authenticator.objects.get(
        user=user_with_recovery_codes, type=Authenticator.Type.RECOVERY_CODES
    ).wrap()
    prev_code = rc.get_unused_codes()[0]

    resp = auth_client.get(reverse("mfa_generate_recovery_codes"))
    assert resp["location"].startswith(reverse("account_reauthenticate"))
    resp = auth_client.post(resp["location"], {"password": user_password})
    assert resp.status_code == HTTPStatus.FOUND
    resp = auth_client.post(resp["location"])
    assert resp["location"] == reverse("mfa_view_recovery_codes")

    rc = Authenticator.objects.get(
        user=user_with_recovery_codes, type=Authenticator.Type.RECOVERY_CODES
    ).wrap()
    assert not rc.validate_code(prev_code)
    assert len(mailoutbox) == 1
    assert "New Recovery Codes Generated" in mailoutbox[0].subject
    assert "A new set of" in mailoutbox[0].body


def test_recovery_codes_login(
    client, user_with_totp, user_with_recovery_codes, user_password
):
    resp = client.post(
        reverse("account_login"),
        {"login": user_with_totp.username, "password": user_password},
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("mfa_authenticate")
    resp = client.get(reverse("mfa_authenticate"))
    assert resp.context["request"].user.is_anonymous
    resp = client.post(reverse("mfa_authenticate"), {"code": "123"})
    assert resp.context["form"].errors == {
        "code": [get_adapter().error_messages["incorrect_code"]]
    }
    rc = Authenticator.objects.get(
        user=user_with_recovery_codes, type=Authenticator.Type.RECOVERY_CODES
    )
    resp = client.post(
        reverse("mfa_authenticate"),
        {"code": rc.wrap().get_unused_codes()[0]},
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == settings.LOGIN_REDIRECT_URL
    assert client.session[AUTHENTICATION_METHODS_SESSION_KEY] == [
        {"method": "password", "at": ANY, "username": user_with_totp.username},
        {
            "method": "mfa",
            "at": ANY,
            "id": ANY,
            "type": Authenticator.Type.RECOVERY_CODES,
        },
    ]



from http import HTTPStatus
from unittest.mock import ANY

from django.urls import reverse

import pytest

from allauth import app_settings as allauth_settings
from allauth.account.authentication import AUTHENTICATION_METHODS_SESSION_KEY
from allauth.account.internal.stagekit import LOGIN_SESSION_KEY
from allauth.account.models import EmailAddress
from allauth.account.stages import LoginByCodeStage


@pytest.fixture
def get_expected_login_code():
    def f(client):
        return client.session[LOGIN_SESSION_KEY]["state"]["stages"][
            LoginByCodeStage.key
        ]["data"]["code"]

    return f


@pytest.fixture
def request_login_by_code(mailoutbox, get_expected_login_code):
    def f(client, email):
        resp = client.get(f"{reverse('account_request_login_code')}?next=/foo")
        assert resp.status_code == HTTPStatus.OK
        assert b'value="/foo"' in resp.content
        resp = client.post(
            reverse("account_request_login_code"), data={"email": email, "next": "/foo"}
        )
        assert resp.status_code == HTTPStatus.FOUND
        assert (
            resp["location"] == f"{reverse('account_confirm_login_code')}?next=%2Ffoo"
        )
        assert len(mailoutbox) == 1
        code = get_expected_login_code(client)
        assert len(code) == 9
        assert code in mailoutbox[0].body
        return code

    return f


def test_login_by_code(client, user, request_login_by_code):
    code = request_login_by_code(client, user.email)
    code_with_ws = f" {code[0:3]} {code[3:]}"
    resp = client.post(
        reverse("account_confirm_login_code"),
        data={"code": code_with_ws, "next": "/foo"},
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert LOGIN_SESSION_KEY not in client.session
    assert resp["location"] == "/foo"
    assert client.session[AUTHENTICATION_METHODS_SESSION_KEY][-1] == {
        "method": "code",
        "email": user.email,
        "at": ANY,
    }


def test_login_by_code_resend_limited(
    client, user, request_login_by_code, settings, get_expected_login_code
):
    settings.ACCOUNT_LOGIN_BY_CODE_SUPPORTS_RESEND = 2
    code = request_login_by_code(client, user.email)
    for i in range(3):
        resp = client.post(
            reverse("account_confirm_login_code"), data={"action": "resend"}
        )

        assert resp.status_code == (
            # Form error if resend not possible
            HTTPStatus.OK
            if i == 2
            # If resend allowed, the resend form is valid and results in a redirect.
            else HTTPStatus.FOUND
        )
        new_code = get_expected_login_code(client)
        if i == 2:
            assert new_code == code
        else:
            assert new_code != code
            code = new_code


def test_login_by_code_resend_uses_new_code(
    client, user, request_login_by_code, settings, get_expected_login_code
):
    settings.ACCOUNT_LOGIN_BY_CODE_SUPPORTS_RESEND = 2
    code = request_login_by_code(client, user.email)
    resp = client.post(reverse("account_confirm_login_code"), data={"action": "resend"})
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("account_confirm_login_code")
    new_code = get_expected_login_code(client)
    assert new_code != code
    resp = client.post(reverse("account_confirm_login_code"), data={"code": new_code})
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == settings.LOGIN_REDIRECT_URL


def test_login_by_code_max_attempts(client, user, request_login_by_code, settings):
    settings.ACCOUNT_LOGIN_BY_CODE_MAX_ATTEMPTS = 2
    request_login_by_code(client, user.email)
    for i in range(3):
        resp = client.post(
            reverse("account_confirm_login_code"), data={"code": "wrong"}
        )
        if i >= 1:
            assert resp.status_code == HTTPStatus.FOUND
            assert resp["location"] == reverse("account_request_login_code")
            assert LOGIN_SESSION_KEY not in client.session
        else:
            assert resp.status_code == HTTPStatus.OK
            assert LOGIN_SESSION_KEY in client.session
            assert resp.context["form"].errors == {"code": ["Incorrect code."]}


def test_login_by_code_unknown_user(mailoutbox, client, db):
    resp = client.post(
        reverse("account_request_login_code"),
        data={"email": "unknown@email.org"},
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("account_confirm_login_code")
    resp = client.post(reverse("account_confirm_login_code"), data={"code": "123456"})


@pytest.mark.parametrize(
    "setting,code_required",
    [
        (True, True),
        ({"password"}, True),
        ({"socialaccount"}, False),
    ],
)
def test_login_by_code_required(
    client, settings, user_factory, password_factory, setting, code_required
):
    password = password_factory()
    user = user_factory(password=password, email_verified=False)
    email_address = EmailAddress.objects.get(email=user.email)
    assert not email_address.verified
    settings.ACCOUNT_LOGIN_BY_CODE_REQUIRED = setting
    resp = client.post(
        reverse("account_login"),
        data={"login": user.username, "password": password},
    )
    assert resp.status_code == HTTPStatus.FOUND
    if code_required:
        assert resp["location"] == reverse("account_confirm_login_code")
        code = client.session[LOGIN_SESSION_KEY]["state"]["stages"][
            LoginByCodeStage.key
        ]["data"]["code"]
        resp = client.get(
            reverse("account_confirm_login_code"),
            data={"login": user.username, "password": password},
        )
        assert resp.status_code == HTTPStatus.OK
        resp = client.post(reverse("account_confirm_login_code"), data={"code": code})
        email_address.refresh_from_db()
        assert email_address.verified
    assert resp["location"] == settings.LOGIN_REDIRECT_URL


def test_login_by_code_redirect(client, user, request_login_by_code):
    request_login_by_code(client, user.email)
    resp = client.get(reverse("account_login"))
    assert resp["location"] == reverse("account_confirm_login_code")


@pytest.mark.parametrize("action", ["", "trust"])
def test_trust_flow(
    client,
    user,
    user_password,
    settings_impacting_urls,
    action,
    settings,
):
    if not allauth_settings.MFA_ENABLED:
        return

    with settings_impacting_urls(
        ACCOUNT_LOGIN_BY_CODE_TRUST_ENABLED=True, ACCOUNT_LOGIN_BY_CODE_REQUIRED=True
    ):

        # Login
        resp = client.post(
            reverse("account_login"),
            {"login": user.username, "password": user_password},
        )
        assert resp.status_code == HTTPStatus.FOUND
        assert resp["location"] == reverse("account_confirm_login_code")

        code = client.session[LOGIN_SESSION_KEY]["state"]["stages"][
            LoginByCodeStage.key
        ]["data"]["code"]

        # Enter code
        resp = client.post(reverse("account_confirm_login_code"), data={"code": code})
        assert resp.status_code == HTTPStatus.FOUND
        assert resp["location"] == reverse("mfa_trust")

        # Indicate trust
        resp = client.post(
            reverse("mfa_trust"),
            {"action": action},
        )
        assert resp["location"] == settings.LOGIN_REDIRECT_URL

        # Sign out
        resp = client.post(
            reverse("account_logout"),
        )
        assert resp.status_code == HTTPStatus.FOUND

        # Sign in
        resp = client.post(
            reverse("account_login"),
            {"login": user.username, "password": user_password},
        )
        assert resp.status_code == HTTPStatus.FOUND
        assert resp["location"] == (
            settings.LOGIN_REDIRECT_URL
            if action == "trust"
            else reverse("account_confirm_login_code")
        )


def test_trust_while_already_register(
    client, user, user_password, settings_impacting_urls, settings, mailoutbox
):
    if not allauth_settings.MFA_ENABLED:
        return

    with settings_impacting_urls(
        ACCOUNT_SIGNUP_FIELDS=["email*", "password*"],
        ACCOUNT_LOGIN_BY_CODE_TRUST_ENABLED=True,
        ACCOUNT_LOGIN_BY_CODE_REQUIRED=True,
        ACCOUNT_PREVENT_ENUMERATION=True,
        ACCOUNT_EMAIL_VERIFICATION="mandatory",
    ):
        resp = client.post(
            reverse("account_signup"),
            {"email": user.email, "password": user_password},
        )
        assert resp.status_code == HTTPStatus.FOUND
        assert resp["location"] == reverse("account_confirm_login_code")
        assert mailoutbox[0].subject == "[example.com] Account Already Exists"



from http import HTTPStatus
from unittest.mock import ANY

import pytest

from allauth.account.signals import user_logged_in
from allauth.headless.base.response import AuthenticationResponse


def test_auth_password_input_error(headless_reverse, client):
    resp = client.post(
        headless_reverse("headless:account:login"),
        data={},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.BAD_REQUEST
    assert resp.json() == {
        "status": HTTPStatus.BAD_REQUEST,
        "errors": [
            {
                "message": "This field is required.",
                "code": "required",
                "param": "username",
            },
            {
                "message": "This field is required.",
                "code": "required",
                "param": "password",
            },
        ],
    }


def test_auth_password_bad_password(headless_reverse, client, user, settings):
    settings.ACCOUNT_LOGIN_METHODS = {"email"}
    resp = client.post(
        headless_reverse("headless:account:login"),
        data={
            "email": user.email,
            "password": "wrong",
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.BAD_REQUEST
    assert resp.json() == {
        "status": HTTPStatus.BAD_REQUEST,
        "errors": [
            {
                "param": "password",
                "message": "The email address and/or password you specified are not correct.",
                "code": "email_password_mismatch",
            }
        ],
    }


def test_auth_password_success(
    client, user, user_password, settings, headless_reverse, headless_client
):
    settings.ACCOUNT_LOGIN_METHODS = {"email", "username"}
    login_resp = client.post(
        headless_reverse("headless:account:login"),
        data={
            "email": user.email,
            "password": user_password,
        },
        content_type="application/json",
    )
    assert login_resp.status_code == HTTPStatus.OK
    session_resp = client.get(
        headless_reverse("headless:account:current_session"),
        content_type="application/json",
    )
    assert session_resp.status_code == HTTPStatus.OK
    for resp in [login_resp, session_resp]:
        extra_meta = {}
        if headless_client == "app" and resp == login_resp:
            # The session is created on first login, and hence the token is
            # exposed only at that moment.
            extra_meta["session_token"] = ANY
        assert resp.json() == {
            "status": HTTPStatus.OK,
            "data": {
                "user": {
                    "id": user.pk,
                    "display": str(user),
                    "email": user.email,
                    "username": user.username,
                    "has_usable_password": True,
                },
                "methods": [
                    {
                        "at": ANY,
                        "email": user.email,
                        "method": "password",
                    }
                ],
            },
            "meta": {"is_authenticated": True, **extra_meta},
        }


@pytest.mark.parametrize(
    "is_active,status_code", [(False, HTTPStatus.UNAUTHORIZED), (True, HTTPStatus.OK)]
)
def test_auth_password_user_inactive(
    client, user, user_password, settings, status_code, is_active, headless_reverse
):
    user.is_active = is_active
    user.save(update_fields=["is_active"])
    resp = client.post(
        headless_reverse("headless:account:login"),
        data={
            "username": user.username,
            "password": user_password,
        },
        content_type="application/json",
    )
    assert resp.status_code == status_code


def test_login_failed_rate_limit(
    client,
    user,
    settings,
    headless_reverse,
    headless_client,
    enable_cache,
):
    settings.ACCOUNT_RATE_LIMITS = {"login_failed": "1/m/ip"}
    settings.ACCOUNT_LOGIN_METHODS = {"email"}
    for attempt in range(2):
        resp = client.post(
            headless_reverse("headless:account:login"),
            data={
                "email": user.email,
                "password": "wrong",
            },
            content_type="application/json",
        )
        assert resp.status_code == HTTPStatus.BAD_REQUEST
        assert resp.json()["errors"] == [
            (
                {
                    "code": "email_password_mismatch",
                    "message": "The email address and/or password you specified are not correct.",
                    "param": "password",
                }
                if attempt == 0
                else {
                    "message": "Too many failed login attempts. Try again later.",
                    "code": "too_many_login_attempts",
                }
            )
        ]


def test_login_rate_limit(
    client,
    user,
    user_password,
    settings,
    headless_reverse,
    headless_client,
    enable_cache,
):
    settings.ACCOUNT_RATE_LIMITS = {"login": "1/m/ip"}
    settings.ACCOUNT_LOGIN_METHODS = {"email"}
    for attempt in range(2):
        resp = client.post(
            headless_reverse("headless:account:login"),
            data={
                "email": user.email,
                "password": user_password,
            },
            content_type="application/json",
        )
        expected_status = HTTPStatus.TOO_MANY_REQUESTS if attempt else HTTPStatus.OK
        assert resp.status_code == expected_status


def test_login_already_logged_in(
    auth_client, user, user_password, settings, headless_reverse
):
    settings.ACCOUNT_LOGIN_METHODS = {"email"}
    resp = auth_client.post(
        headless_reverse("headless:account:login"),
        data={
            "email": user.email,
            "password": user_password,
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.CONFLICT


def test_custom_post_login_response(
    settings, client, headless_reverse, user, user_password
):
    settings.ACCOUNT_LOGIN_METHODS = {"email"}

    def on_user_logged_in(**kwargs):
        response = kwargs["response"]
        assert isinstance(response, AuthenticationResponse)

    user_logged_in.connect(on_user_logged_in)
    try:
        login_resp = client.post(
            headless_reverse("headless:account:login"),
            data={
                "email": user.email,
                "password": user_password,
            },
            content_type="application/json",
        )
        assert login_resp.status_code == HTTPStatus.OK
    finally:
        user_logged_in.disconnect(on_user_logged_in)



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.asana.provider import AsanaProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class AsanaTests(OAuth2TestsMixin, TestCase):
    provider_id = AsanaProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """
{"data": {"photo": null, "workspaces": [{"id": 31337, "name": "example.com"},
{"id": 3133777, "name": "Personal Projects"}], "email": "test@example.com",
"name": "Test Name", "gid": 43748387}}""",
        )

    def get_expected_to_str(self):
        return "test@example.com"



from django.contrib.auth.hashers import PBKDF2PasswordHasher

from tests.projects.common.settings import HEADLESS_JWT_PRIVATE_KEY  # noqa
from tests.projects.common.settings import IDP_OIDC_PRIVATE_KEY  # noqa
from tests.projects.common.settings import TEMPLATES  # noqa
from tests.projects.common.settings import INSTALLED_SOCIALACCOUNT_APPS


SECRET_KEY = "psst"
SITE_ID = 1
ALLOWED_HOSTS = (
    "testserver",
    "example.com",
)
USE_I18N = False
USE_TZ = True

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
        "USER": "",
        "PASSWORD": "",
        "HOST": "",
        "PORT": "",
    }
}

ROOT_URLCONF = "tests.projects.login_required_mw.urls"
LOGIN_URL = "/accounts/login/"

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
    }
}

MIDDLEWARE = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "tests.projects.login_required_mw.middleware.CustomLoginRequiredMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "allauth.account.middleware.AccountMiddleware",
)

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.admin",
    "django.contrib.humanize",
    "allauth",
    "allauth.account",
    "allauth.mfa",
    "allauth.usersessions",
    "allauth.headless",
    "allauth.idp.oidc",
) + INSTALLED_SOCIALACCOUNT_APPS

AUTHENTICATION_BACKENDS = (
    "django.contrib.auth.backends.ModelBackend",
    "allauth.account.auth_backends.AuthenticationBackend",
)

STATIC_ROOT = "/tmp/"  # Dummy
STATIC_URL = "/static/"


class MyPBKDF2PasswordHasher(PBKDF2PasswordHasher):
    """
    A subclass of PBKDF2PasswordHasher that uses 1 iteration.

    This is for test purposes only. Never use anywhere else.
    """

    iterations = 1


PASSWORD_HASHERS = [
    "tests.projects.login_required_mw.settings.MyPBKDF2PasswordHasher",
    "django.contrib.auth.hashers.PBKDF2PasswordHasher",
    "django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher",
    "django.contrib.auth.hashers.Argon2PasswordHasher",
    "django.contrib.auth.hashers.BCryptSHA256PasswordHasher",
]


SOCIALACCOUNT_QUERY_EMAIL = True
SOCIALACCOUNT_PROVIDERS = {
    "openid_connect": {
        "APPS": [
            {
                "provider_id": "unittest-server",
                "name": "Unittest Server",
                "client_id": "Unittest client_id",
                "client_secret": "Unittest client_secret",
                "settings": {
                    "server_url": "https://unittest.example.com",
                },
            },
            {
                "provider_id": "other-server",
                "name": "Other Example Server",
                "client_id": "other client_id",
                "client_secret": "other client_secret",
                "settings": {
                    "server_url": "https://other.example.com",
                },
            },
        ],
    }
}

ACCOUNT_LOGIN_BY_CODE_ENABLED = True
ACCOUNT_ADAPTER = "tests.projects.common.adapters.AccountAdapter"

MFA_SUPPORTED_TYPES = ["totp", "webauthn", "recovery_codes"]
MFA_PASSKEY_LOGIN_ENABLED = True
MFA_PASSKEY_SIGNUP_ENABLED = True

HEADLESS_SERVE_SPECIFICATION = True



from pathlib import Path

from django.contrib.auth.hashers import PBKDF2PasswordHasher


SECRET_KEY = "psst"
SITE_ID = 1
ALLOWED_HOSTS = (
    "testserver",
    "example.com",
)
USE_I18N = False
USE_TZ = True

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
        "USER": "",
        "PASSWORD": "",
        "HOST": "",
        "PORT": "",
    }
}

ROOT_URLCONF = "tests.projects.account_only.urls"
LOGIN_URL = "/accounts/login/"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [Path(__file__).parent / "templates"],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
    }
}

MIDDLEWARE = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "allauth.account.middleware.AccountMiddleware",
)

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.admin",
    "django.contrib.humanize",
    "allauth",
    "allauth.account",
)

AUTHENTICATION_BACKENDS = (
    "django.contrib.auth.backends.ModelBackend",
    "allauth.account.auth_backends.AuthenticationBackend",
)

STATIC_ROOT = "/tmp/"  # Dummy
STATIC_URL = "/static/"


class MyPBKDF2PasswordHasher(PBKDF2PasswordHasher):
    """
    A subclass of PBKDF2PasswordHasher that uses 1 iteration.

    This is for test purposes only. Never use anywhere else.
    """

    iterations = 1


PASSWORD_HASHERS = [
    "tests.projects.headless_only.settings.MyPBKDF2PasswordHasher",
    "django.contrib.auth.hashers.PBKDF2PasswordHasher",
    "django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher",
    "django.contrib.auth.hashers.Argon2PasswordHasher",
    "django.contrib.auth.hashers.BCryptSHA256PasswordHasher",
]


ACCOUNT_LOGIN_BY_CODE_ENABLED = True
ACCOUNT_ADAPTER = "tests.projects.common.adapters.AccountAdapter"



from http import HTTPStatus

from django.test.client import Client
from django.urls import reverse


def test_app_session_gone(db, user):
    # intentionally use a vanilla Django test client
    client = Client()
    # Force login, creates a Django session
    client.force_login(user)
    # That Django session should not play any role.
    resp = client.get(
        reverse("headless:app:account:current_session"), HTTP_X_SESSION_TOKEN="gone"
    )
    assert resp.status_code == HTTPStatus.GONE


def test_logout(auth_client, headless_reverse):
    # That Django session should not play any role.
    resp = auth_client.get(headless_reverse("headless:account:current_session"))
    assert resp.status_code == HTTPStatus.OK
    resp = auth_client.delete(headless_reverse("headless:account:current_session"))
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    resp = auth_client.get(headless_reverse("headless:account:current_session"))
    assert resp.status_code in [HTTPStatus.UNAUTHORIZED, HTTPStatus.GONE]


def test_logout_no_token(app_client, user):
    app_client.force_login(user)
    resp = app_client.get(reverse("headless:app:account:current_session"))
    assert resp.status_code == HTTPStatus.OK
    resp = app_client.delete(reverse("headless:app:account:current_session"))
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    assert "session_token" not in resp.json()["meta"]



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.kakao.provider import KakaoProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class KakaoTests(OAuth2TestsMixin, TestCase):
    provider_id = KakaoProvider.id

    kakao_data = """
        {
            "id": 123456789,
            "connected_at": "2022-04-11T01:45:28Z",
            "kakao_account": {
                "profile_nickname_needs_agreement": false,
                "profile_image_needs_agreement": false,
                "profile": {
                    "nickname": "홍길동",
                    "thumbnail_image_url": "http://yyy.kakao.com/.../img_110x110.jpg",
                    "profile_image_url": "http://yyy.kakao.com/dn/.../img_640x640.jpg",
                    "is_default_image":false,
                    "is_default_nickname": false
                },
                "name_needs_agreement":false,
                "name":"홍길동",
                "email_needs_agreement":false,
                "is_email_valid": true,
                "is_email_verified": true,
                "email": "sample@sample.com",
                "age_range_needs_agreement":false,
                "age_range":"20~29",
                "birthyear_needs_agreement": false,
                "birthyear": "2002",
                "birthday_needs_agreement":false,
                "birthday":"1130",
                "birthday_type":"SOLAR",
                "gender_needs_agreement":false,
                "gender":"female",
                "phone_number_needs_agreement": false,
                "phone_number": "+82 010-1234-5678",
                "ci_needs_agreement": false,
                "ci": "CI",
                "ci_authenticated_at": "2019-03-11T11:25:22Z"
            },
            "properties":{
                "CUSTOM_PROPERTY_KEY": "CUSTOM_PROPERTY_VALUE"
            },
            "for_partner": {
                "uuid": "UUID"
            }
        }
    """

    def get_expected_to_str(self):
        return "sample@sample.com"

    def get_mocked_response(self, data=None):
        if data is None:
            data = self.kakao_data
        return MockedResponse(HTTPStatus.OK, data)



import uuid
from unittest.mock import patch

from django.contrib import messages
from django.contrib.auth import get_user_model
from django.contrib.messages.api import get_messages
from django.contrib.messages.middleware import MessageMiddleware
from django.contrib.sessions.middleware import SessionMiddleware
from django.core import mail, validators
from django.core.exceptions import ValidationError
from django.template import Context, Template
from django.test.client import RequestFactory
from django.test.utils import override_settings
from django.urls import reverse

import pytest

import allauth.app_settings
from allauth.account.adapter import get_adapter
from allauth.account.models import EmailAddress
from allauth.account.utils import (
    filter_users_by_username,
    url_str_to_user_pk,
    user_pk_to_url_str,
    user_username,
)
from allauth.core import context

from .test_models import UUIDUser


test_username_validators = [
    validators.RegexValidator(regex=r"^[a-c]+$", message="not abc")
]


def test_url_str_to_pk_identifies_UUID_as_stringlike(db):
    with patch("allauth.account.utils.get_user_model") as mocked_gum:
        mocked_gum.return_value = UUIDUser
        user_id = uuid.uuid4().hex
        assert url_str_to_user_pk(user_id) == uuid.UUID(user_id)


def test_pk_to_url_string_identifies_UUID_as_stringlike():
    with patch("allauth.account.utils.get_user_model") as mocked_gum:
        mocked_gum.return_value = UUIDUser
        user = UUIDUser(is_active=True, email="john@example.com", username="john")
        assert user_pk_to_url_str(user) == user.pk.hex


@override_settings(ACCOUNT_PRESERVE_USERNAME_CASING=False)
def test_username_lower_cased():
    user = get_user_model()()
    user_username(user, "CamelCase")
    assert user_username(user) == "camelcase"
    # TODO: Actually test something
    filter_users_by_username("CamelCase", "FooBar")


@override_settings(ACCOUNT_PRESERVE_USERNAME_CASING=True)
def test_username_case_preserved():
    user = get_user_model()()
    user_username(user, "CamelCase")
    assert user_username(user) == "CamelCase"
    # TODO: Actually test something
    filter_users_by_username("camelcase", "foobar")


def test_user_display():
    user = get_user_model()(username="john<br/>doe")
    expected_name = "john&lt;br/&gt;doe"
    templates = [
        "{% load account %}{% user_display user %}",
        "{% load account %}{% user_display user as x %}{{ x }}",
    ]
    for template in templates:
        t = Template(template)
        content = t.render(Context({"user": user}))
        assert content == expected_name


def test_message_escaping(db):
    request = RequestFactory().get("/")
    SessionMiddleware(lambda request: None).process_request(request)
    MessageMiddleware(lambda request: None).process_request(request)
    user = get_user_model()()
    user_username(user, "'<8")
    context = {"user": user}
    get_adapter().add_message(
        request, messages.SUCCESS, "account/messages/logged_in.txt", context
    )
    msgs = get_messages(request)
    actual_message = msgs._queued_messages[0].message
    assert user.username in actual_message, actual_message


def test_email_escaping(db):
    site_name = "testserver"
    if allauth.app_settings.SITES_ENABLED:
        from django.contrib.sites.models import Site

        site = Site.objects.get_current()
        site.name = site_name = '<enc&"test>'
        site.save()
    u = get_user_model().objects.create(username="test", email="user@example.com")
    request = RequestFactory().get("/")
    SessionMiddleware(lambda request: None).process_request(request)
    MessageMiddleware(lambda request: None).process_request(request)
    EmailAddress.objects.add_email(request, u, u.email, confirm=True)
    assert mail.outbox[0].subject[1:].startswith(site_name)


@override_settings(
    ACCOUNT_USERNAME_VALIDATORS="tests.apps.account.test_utils.test_username_validators"
)
def test_username_validator(db):
    get_adapter().clean_username("abc")
    with pytest.raises(ValidationError):
        get_adapter().clean_username("def")


@override_settings(ALLOWED_HOSTS=["allowed_host", "testserver"])
def test_is_safe_url_no_wildcard():
    with context.request_context(RequestFactory().get("/")):
        assert get_adapter().is_safe_url("http://allowed_host/")
        assert not get_adapter().is_safe_url("http://other_host/")


def test_is_safe_url_subdomain(settings):
    settings.ALLOWED_HOSTS = [".example.com", "testserver"]
    with context.request_context(RequestFactory().get("/")):
        assert get_adapter().is_safe_url("http://bla.example.com")
        assert get_adapter().is_safe_url("http://example.com")
        assert not get_adapter().is_safe_url("http://not-example.com")


@override_settings(ALLOWED_HOSTS=["*"])
def test_is_safe_url_wildcard():
    with context.request_context(RequestFactory().get("/")):
        assert get_adapter().is_safe_url("http://foobar.com/")
        assert get_adapter().is_safe_url("http://other_host/")


@override_settings(ALLOWED_HOSTS=["allowed_host", "testserver"])
def test_is_safe_url_relative_path():
    with context.request_context(RequestFactory().get("/")):
        assert get_adapter().is_safe_url("/foo/bar")


def test_redirect_noreversematch(auth_client):
    # We used to call `django.shortcuts.redirect()` as is, but that one throws a
    # `NoReverseMatch`, resulting in 500s.
    resp = auth_client.post(f"{reverse('account_logout')}?next=badurlname")
    assert resp["location"] == "/badurlname"



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.auth0.provider import Auth0Provider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class Auth0Tests(OAuth2TestsMixin, TestCase):
    provider_id = Auth0Provider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """
            {
                "picture": "https://secure.gravatar.com/avatar/123",
                "email": "mr.bob@your.Auth0.server.example.com",
                "id": 2,
                "sub": 2,
                "identities": [],
                "name": "Mr Bob"
            }
        """,
        )

    def get_expected_to_str(self):
        return "mr.bob@your.Auth0.server.example.com"



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.sharefile.provider import ShareFileProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class ShareFileTests(OAuth2TestsMixin, TestCase):
    provider_id = ShareFileProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """
{
  "Id": "123",
  "Email":"user.one@domain.com",
  "FirstName":"Name",
  "LastName":"Last Name",
  "Company":"Company",
  "DefaultZone":
  {
    "Id":"zoneid"
  }
}         """,
        )

    def get_expected_to_str(self):
        return "user.one@domain.com"



from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class AngelListOAuth2Adapter(OAuth2Adapter):
    provider_id = "angellist"
    access_token_url = "https://angel.co/api/oauth/token/"  # nosec
    authorize_url = "https://angel.co/api/oauth/authorize/"
    profile_url = "https://api.angel.co/1/me/"
    supports_state = False

    def complete_login(self, request, app, token, **kwargs):
        with get_adapter().get_requests_session() as sess:
            resp = sess.get(self.profile_url, params={"access_token": token.token})
            extra_data = resp.json()
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(AngelListOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(AngelListOAuth2Adapter)



from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)

from .constants import FXA_OAUTH_ENDPOINT, FXA_PROFILE_ENDPOINT, PROVIDER_ID


class FirefoxAccountsOAuth2Adapter(OAuth2Adapter):
    provider_id = PROVIDER_ID
    access_token_url = f"{FXA_OAUTH_ENDPOINT}/token"
    authorize_url = f"{FXA_OAUTH_ENDPOINT}/authorization"
    profile_url = f"{FXA_PROFILE_ENDPOINT}/profile"

    def complete_login(self, request, app, token, **kwargs):
        headers = {"Authorization": f"Bearer {token.token}"}
        with get_adapter().get_requests_session() as sess:
            resp = sess.get(self.profile_url, headers=headers)
            extra_data = resp.json()
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(FirefoxAccountsOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(FirefoxAccountsOAuth2Adapter)



from http import HTTPStatus
from unittest.mock import ANY, patch

from django.contrib.auth.models import User

import pytest

from allauth.account.models import EmailAddress, EmailConfirmationHMAC
from allauth.account.signals import user_logged_in
from allauth.headless.base.response import AuthenticationResponse
from allauth.headless.constants import Flow


def test_signup(
    db,
    client,
    email_factory,
    password_factory,
    settings,
    headless_reverse,
    headless_client,
):
    def on_user_logged_in(**kwargs):
        response = kwargs["response"]
        assert isinstance(response, AuthenticationResponse)

    user_logged_in.connect(on_user_logged_in)
    try:
        resp = client.post(
            headless_reverse("headless:account:signup"),
            data={
                "username": "wizard",
                "email": email_factory(),
                "password": password_factory(),
            },
            content_type="application/json",
        )
        assert resp.status_code == HTTPStatus.OK
        assert User.objects.filter(username="wizard").exists()
    finally:
        user_logged_in.disconnect(on_user_logged_in)


def test_signup_with_email_verification(
    db,
    client,
    email_factory,
    password_factory,
    settings,
    headless_reverse,
    headless_client,
):
    settings.ACCOUNT_EMAIL_VERIFICATION = "mandatory"
    settings.ACCOUNT_USERNAME_REQUIRED = False
    settings.ACCOUNT_LOGIN_ON_EMAIL_CONFIRMATION = True
    email = email_factory()
    resp = client.post(
        headless_reverse("headless:account:signup"),
        data={
            "email": email,
            "password": password_factory(),
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    assert User.objects.filter(email=email).exists()
    data = resp.json()
    flow = next(f for f in data["data"]["flows"] if f.get("is_pending"))
    assert flow["id"] == "verify_email"
    addr = EmailAddress.objects.get(email=email)
    key = EmailConfirmationHMAC(addr).key
    resp = client.get(
        headless_reverse("headless:account:verify_email"),
        HTTP_X_EMAIL_VERIFICATION_KEY=key,
    )
    assert resp.status_code == HTTPStatus.OK
    assert resp.json() == {
        "data": {
            "email": email,
            "user": ANY,
        },
        "meta": {"is_authenticating": True},
        "status": HTTPStatus.OK,
    }
    resp = client.post(
        headless_reverse("headless:account:verify_email"),
        data={"key": key},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.OK
    data = resp.json()
    assert data["meta"]["is_authenticated"]
    addr.refresh_from_db()
    assert addr.verified


@pytest.mark.parametrize("email_verification_by_code_enabled", (False, True))
def test_signup_prevent_enumeration(
    db,
    client,
    email_factory,
    password_factory,
    settings,
    headless_reverse,
    headless_client,
    user,
    mailoutbox,
    email_verification_by_code_enabled,
):
    settings.ACCOUNT_EMAIL_VERIFICATION_BY_CODE_ENABLED = (
        email_verification_by_code_enabled
    )
    settings.ACCOUNT_EMAIL_VERIFICATION = "mandatory"
    settings.ACCOUNT_USERNAME_REQUIRED = False
    settings.ACCOUNT_PREVENT_ENUMERATION = True
    resp = client.post(
        headless_reverse("headless:account:signup"),
        data={
            "email": user.email,
            "password": password_factory(),
        },
        content_type="application/json",
    )
    assert len(mailoutbox) == 1
    assert "an account using that email address already exists" in mailoutbox[0].body
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    data = resp.json()
    assert [f for f in data["data"]["flows"] if f["id"] == Flow.VERIFY_EMAIL][0][
        "is_pending"
    ]
    resp = client.get(headless_reverse("headless:account:current_session"))
    data = resp.json()
    assert [f for f in data["data"]["flows"] if f["id"] == Flow.VERIFY_EMAIL][0][
        "is_pending"
    ]
    resp = client.post(
        headless_reverse("headless:account:verify_email"),
        data={"key": "wrong"},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.BAD_REQUEST


def test_signup_rate_limit(
    db,
    client,
    email_factory,
    password_factory,
    settings,
    headless_reverse,
    enable_cache,
    headless_client,
):
    settings.ACCOUNT_RATE_LIMITS = {"signup": "1/m/ip"}
    for attempt in range(2):
        resp = client.post(
            headless_reverse("headless:account:signup"),
            data={
                "username": f"wizard{attempt}",
                "email": email_factory(),
                "password": password_factory(),
            },
            content_type="application/json",
        )
        expected_status = HTTPStatus.TOO_MANY_REQUESTS if attempt else HTTPStatus.OK
        assert resp.status_code == expected_status
        assert resp.json()["status"] == expected_status


def test_signup_closed(
    db,
    client,
    email_factory,
    password_factory,
    settings,
    headless_reverse,
    headless_client,
):
    with patch(
        "allauth.account.adapter.DefaultAccountAdapter.is_open_for_signup"
    ) as iofs:
        iofs.return_value = False
        resp = client.post(
            headless_reverse("headless:account:signup"),
            data={
                "username": "wizard",
                "email": email_factory(),
                "password": password_factory(),
            },
            content_type="application/json",
        )
    assert resp.status_code == HTTPStatus.FORBIDDEN
    assert not User.objects.filter(username="wizard").exists()


def test_signup_while_logged_in(
    db,
    auth_client,
    email_factory,
    password_factory,
    settings,
    headless_reverse,
    headless_client,
):
    resp = auth_client.post(
        headless_reverse("headless:account:signup"),
        data={
            "username": "wizard",
            "email": email_factory(),
            "password": password_factory(),
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.CONFLICT
    assert resp.json() == {"status": HTTPStatus.CONFLICT}


def test_signup_without_password(
    db,
    client,
    email_factory,
    password_factory,
    headless_reverse,
    headless_client,
    settings_impacting_urls,
):
    with settings_impacting_urls(
        ACCOUNT_LOGIN_BY_CODE_ENABLED=True,
        ACCOUNT_EMAIL_VERIFICATION="mandatory",
        ACCOUNT_EMAIL_VERIFICATION_BY_CODE_ENABLED=True,
        ACCOUNT_SIGNUP_FIELDS=["email*", "password1"],
    ):
        email = email_factory()
        resp = client.post(
            headless_reverse("headless:account:signup"),
            data={
                "username": "wizard",
                "email": email,
            },
            content_type="application/json",
        )
        assert resp.status_code == HTTPStatus.UNAUTHORIZED
        user = User.objects.get(email=email)
        assert not user.check_password("")



import pytest

from allauth.account.internal import emailkit


@pytest.mark.parametrize(
    "email,valid",
    [
        (
            "this.email.address.is.a.bit.too.long.but.should.still.validate@example.com",
            True,
        ),
        (
            ("x" * 300)
            + "this.email.address.is.a.bit.too.long.but.should.still.validate@example.com",
            False,
        ),
    ],
)
def test_email_validation(email, valid):
    clean_email = emailkit.valid_email_or_none(email)
    if valid:
        assert clean_email == email
    else:
        assert clean_email is None



from http import HTTPStatus

from django.test import TestCase
from django.test.client import RequestFactory
from django.urls import reverse

from allauth.socialaccount.models import SocialToken
from allauth.socialaccount.providers.oauth2.client import OAuth2Error
from allauth.socialaccount.providers.twitch.provider import TwitchProvider
from allauth.socialaccount.providers.twitch.views import TwitchOAuth2Adapter
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse, mocked_response


class TwitchTests(OAuth2TestsMixin, TestCase):
    provider_id = TwitchProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """
        {
          "data": [{
            "id": "44322889",
            "login": "dallas",
            "display_name": "dallas",
            "type": "staff",
            "broadcaster_type": "",
            "description": "Just a gamer playing games and chatting. :)",
            "profile_image_url": "https://static-cdn.jtvnw.net/jtv_user_pictures/dallas-profile_image-1a2c906ee2c35f12-300x300.png",
            "offline_image_url": "https://static-cdn.jtvnw.net/jtv_user_pictures/dallas-channel_offline_image-1a2c906ee2c35f12-1920x1080.png",
            "view_count": 191836881,
            "email": "login@provider.com"
          }]
        }
        """,
        )  # noqa

    def get_expected_to_str(self):
        return "dallas"

    def test_response_over_400_raises_OAuth2Error(self):
        resp_mock = MockedResponse(HTTPStatus.BAD_REQUEST, '{"error": "Invalid token"}')
        expected_error = "Twitch API Error: Invalid token ()"

        self.check_for_error(resp_mock, expected_error)

    def test_empty_or_missing_data_key_raises_OAuth2Error(self):
        resp_mock = MockedResponse(HTTPStatus.OK, '{"data": []}')
        expected_error = "Invalid data from Twitch API: {'data': []}"

        self.check_for_error(resp_mock, expected_error)

        resp_mock = MockedResponse(HTTPStatus.OK, '{"missing_data": "key"}')
        expected_error = "Invalid data from Twitch API: {'missing_data': 'key'}"

        self.check_for_error(resp_mock, expected_error)

    def test_missing_twitch_id_raises_OAuth2Error(self):
        resp_mock = MockedResponse(
            HTTPStatus.OK, '{"data": [{"login": "fake_twitch"}]}'
        )
        expected_error = "Invalid data from Twitch API: {'login': 'fake_twitch'}"

        self.check_for_error(resp_mock, expected_error)

    def check_for_error(self, resp_mock, expected_error):
        with self.assertRaises(OAuth2Error) as error_ctx:
            self._run_just_complete_login(resp_mock)

        self.assertEqual(str(error_ctx.exception).replace("u", ""), expected_error)

    def _run_just_complete_login(self, resp_mock):
        """
        Helper function for checking that Error cases are
        handled correctly. Running only `complete_login` means
        we can check that the specific errors are raised before
        they are caught and rendered to generic error HTML
        """
        request = RequestFactory().get(
            reverse(f"{self.provider.id}_login"),
            {"process": "login"},
        )
        adapter = TwitchOAuth2Adapter(request)
        app = adapter.get_provider().app
        token = SocialToken(token="this-is-my-fake-token")

        with mocked_response(resp_mock):
            adapter = TwitchOAuth2Adapter(request)
            adapter.complete_login(request, app, token)



from django.urls import include, path


urlpatterns = [
    path("accounts/", include("allauth.urls")),
    path("_allauth/", include("allauth.headless.urls")),
    path("", include("tests.projects.common.urls")),
]



import time
from http import HTTPStatus
from urllib.parse import parse_qsl, quote, urlencode

from django.core.exceptions import ImproperlyConfigured

import jwt

from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.client import OAuth2Client, OAuth2Error


def jwt_encode(*args, **kwargs):
    resp = jwt.encode(*args, **kwargs)
    if isinstance(resp, bytes):
        # For PyJWT <2
        resp = resp.decode("utf-8")
    return resp


class Scope:
    EMAIL = "email"
    NAME = "name"


class AppleOAuth2Client(OAuth2Client):
    """
    Custom client because `Sign In With Apple`:
        * requires `response_mode` field in redirect_url
        * requires special `client_secret` as JWT
    """

    def generate_client_secret(self) -> str:
        """Create a JWT signed with an apple provided private key"""
        now = int(time.time())
        app = get_adapter(self.request).get_app(self.request, "apple")
        if not app.key:
            raise ImproperlyConfigured("Apple 'key' missing")
        certificate_key = app.settings.get("certificate_key")
        if not certificate_key:
            raise ImproperlyConfigured("Apple 'certificate_key' missing")
        claims = {
            "iss": app.key,
            "aud": "https://appleid.apple.com",
            "sub": self.get_client_id(),
            "iat": now,
            "exp": now + 60 * 60,
        }
        headers = {"kid": self.consumer_secret, "alg": "ES256"}
        client_secret = jwt_encode(
            payload=claims, key=certificate_key, algorithm="ES256", headers=headers
        )
        return client_secret

    def get_client_id(self) -> str:
        """We support multiple client_ids, but use the first one for api calls"""
        return self.consumer_key.split(",")[0]

    def get_access_token(self, code, pkce_code_verifier=None):
        url = self.access_token_url
        client_secret = self.generate_client_secret()
        data = {
            "client_id": self.get_client_id(),
            "code": code,
            "grant_type": "authorization_code",
            "redirect_uri": self.callback_url,
            "client_secret": client_secret,
        }
        if pkce_code_verifier:
            data["code_verifier"] = pkce_code_verifier
        self._strip_empty_keys(data)
        with get_adapter().get_requests_session() as sess:
            resp = sess.request(
                self.access_token_method, url, data=data, headers=self.headers
            )
        access_token = None
        if resp.status_code in [HTTPStatus.OK, HTTPStatus.CREATED]:
            try:
                access_token = resp.json()
            except ValueError:
                access_token = dict(parse_qsl(resp.text))
        if not access_token or "access_token" not in access_token:
            raise OAuth2Error(f"Error retrieving access token: {resp.content}")
        return access_token

    def get_redirect_url(self, authorization_url, scope, extra_params) -> str:
        scope = self.scope_delimiter.join(set(scope))
        params = {
            "client_id": self.get_client_id(),
            "redirect_uri": self.callback_url,
            "response_mode": "form_post",
            "scope": scope,
            "response_type": "code id_token",
        }
        if self.state:
            params["state"] = self.state
        params.update(extra_params)
        return f"{authorization_url}?{urlencode(params, quote_via=quote)}"



from django.test import TestCase

import pytest

from allauth.socialaccount.models import SocialAccount
from allauth.socialaccount.providers.openid_connect.provider import (
    OpenIDConnectProviderAccount,
)
from tests.apps.socialaccount.base import OpenIDConnectTests


class OpenIDConnectFetchUserInfoTests(OpenIDConnectTests, TestCase):
    provider_id = "oidc-server"


class OpenIDConnectUseIDTokenTests(OpenIDConnectTests, TestCase):
    provider_id = "other-oidc-server"

    def setup_provider(self):
        super().setup_provider()
        self.app.settings["fetch_userinfo"] = False
        self.app.save()
        self.extra_data = self.id_token


@pytest.mark.parametrize(
    "extra_data,expected_to_str",
    [
        ({"username": "compatpre6511"}, "compatpre6511"),
        ({"id_token": {"username": "idtokusr"}}, "idtokusr"),
        ({"userinfo": {"username": "userinfousr"}}, "userinfousr"),
    ],
)
def test_socialaccount_extra_data(extra_data, expected_to_str):
    sa = SocialAccount()
    sa.extra_data = extra_data
    assert OpenIDConnectProviderAccount(sa).to_str() == expected_to_str



from unittest import mock

from allauth.mfa import app_settings
from allauth.mfa.totp.internal.auth import (
    format_hotp_value,
    generate_totp_secret,
    hotp_value,
    validate_totp_code,
    yield_hotp_counters_from_time,
)


@mock.patch("time.time", mock.MagicMock(return_value=1731948631))
def test_totp_counters_from_time():
    app_settings.TOTP_TOLERANCE = 0
    counters = list(yield_hotp_counters_from_time())
    assert len(counters) == 1


@mock.patch("time.time", mock.MagicMock(return_value=1731948631))
def test_totp_counters_from_time_with_tolerance():
    app_settings.TOTP_TOLERANCE = 1
    counters = list(yield_hotp_counters_from_time())
    assert len(counters) == 3


@mock.patch("time.time", mock.MagicMock(return_value=1731948631))
def test_validate_with_tolerance():
    app_settings.TOTP_TOLERANCE = 1
    test_secret = generate_totp_secret()
    expected_value = format_hotp_value(hotp_value(test_secret, 57731621))
    assert validate_totp_code(test_secret, expected_value)

    before_value = format_hotp_value(hotp_value(test_secret, 57731620))
    assert validate_totp_code(test_secret, before_value)

    after_value = format_hotp_value(hotp_value(test_secret, 57731622))
    assert validate_totp_code(test_secret, after_value)

    two_before_value = format_hotp_value(hotp_value(test_secret, 57731619))
    assert not validate_totp_code(test_secret, two_before_value)

    two_after_value = format_hotp_value(hotp_value(test_secret, 57731623))
    assert not validate_totp_code(test_secret, two_after_value)



from allauth.headless.constants import Client
from allauth.headless.contrib.rest_framework.authentication import (
    XSessionTokenAuthentication,
)


def test_authenticate(rf, user, headless_client, auth_client):
    if headless_client == Client.BROWSER:
        return
    request = rf.get("/", HTTP_X_SESSION_TOKEN=auth_client.session_token)
    auth_user, session = XSessionTokenAuthentication().authenticate(request)
    assert auth_user.pk == user.pk


def test_invalid_authentication(rf, user, headless_client, auth_client):
    if headless_client == Client.BROWSER:
        return
    request = rf.get("/", HTTP_X_SESSION_TOKEN="wrong")
    result = XSessionTokenAuthentication().authenticate(request)
    assert result is None



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.models import SocialAccount
from allauth.socialaccount.providers.discogs.provider import DiscogsProvider
from tests.apps.socialaccount.base import OAuthTestsMixin
from tests.mocking import MockedResponse


class DiscogsTests(OAuthTestsMixin, TestCase):
    provider_id = DiscogsProvider.id

    def get_mocked_response(self):
        return [
            MockedResponse(
                HTTPStatus.OK,
                r'{"id": 5788917, "username": "veljkoj", '
                r'"resource_url": "https://api.discogs.com/users/veljkoj", '
                r'"consumer_name": "myapp"}',
            )
        ]

    def get_expected_to_str(self):
        return "veljkoj"

    def test_login(self):
        super().test_login()
        account = SocialAccount.objects.get(uid="5788917")
        discogs_account = account.get_provider_account()

        self.assertEqual(discogs_account.get_username(), "veljkoj")
        self.assertEqual(
            discogs_account.get_profile_url(),
            r"https://api.discogs.com/users/veljkoj",
        )
        self.assertEqual(discogs_account.to_str(), "veljkoj")



import uuid

from django.contrib.auth.models import AbstractUser
from django.db import models

from allauth.account.models import EmailAddress


class UUIDUser(AbstractUser):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    class Meta(AbstractUser.Meta):  # type: ignore[name-defined]
        swappable = "AUTH_USER_MODEL"
        app_label = "dummy"


def test_add_new_email(rf, user, settings):
    settings.ACCOUNT_CHANGE_EMAIL = True
    request = rf.get("/")
    assert EmailAddress.objects.filter(user=user).count() == 1
    new_email = EmailAddress.objects.add_new_email(
        request, user, "new@email.org", send_verification=False
    )
    assert not new_email.verified
    assert not new_email.primary
    assert EmailAddress.objects.filter(user=user).count() == 2
    EmailAddress.objects.add_new_email(
        request, user, "new2@email.org", send_verification=False
    )
    assert EmailAddress.objects.filter(user=user).count() == 2
    assert not EmailAddress.objects.filter(pk=new_email.pk).exists()
    assert EmailAddress.objects.filter(email="new2@email.org", verified=False).exists()



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.lichess.provider import LichessProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class LichessTests(OAuth2TestsMixin, TestCase):
    provider_id = LichessProvider.id

    def get_mocked_response(self):
        return [
            MockedResponse(
                HTTPStatus.OK,
                """
{
  "id": "george",
  "url": "https://lichess.org/@/george",
  "count": {
    "ai": 0,
    "me": 0,
    "all": 0,
    "win": 0,
    "draw": 0,
    "loss": 0,
    "winH": 0,
    "drawH": 0,
    "lossH": 0,
    "rated": 0,
    "import": 0,
    "playing": 0,
    "bookmark": 0
  },
  "perfs": {
    "blitz": {
      "rd": 500,
      "prog": 0,
      "prov": true,
      "games": 0,
      "rating": 1500
    },
    "rapid": {
      "rd": 500,
      "prog": 0,
      "prov": true,
      "games": 0,
      "rating": 1500
    },
    "bullet": {
      "rd": 500,
      "prog": 0,
      "prov": true,
      "games": 0,
      "rating": 1500
    },
    "classical": {
      "rd": 500,
      "prog": 0,
      "prov": true,
      "games": 0,
      "rating": 1500
    },
    "correspondence": {
      "rd": 500,
      "prog": 0,
      "prov": true,
      "games": 0,
      "rating": 1500
    }
  },
  "seenAt": 1713837454330,
  "blocking": false,
  "playTime": {
    "tv": 0,
    "total": 0
  },
  "username": "george",
  "createdAt": 1713837409417,
  "following": false,
  "followable": true,
  "followsYou": false
}
""",
            ),
            MockedResponse(HTTPStatus.OK, """{"email":"george@example.com"}"""),
        ]

    def get_expected_to_str(self):
        return "george"



from unittest.mock import Mock

from django.contrib.auth.models import AnonymousUser
from django.test.utils import override_settings

import pytest

from allauth.usersessions.middleware import UserSessionsMiddleware
from allauth.usersessions.models import UserSession
from allauth.usersessions.signals import session_client_changed


def test_mw_without_request_user(rf, db, settings):
    settings.USERSESSIONS_TRACK_ACTIVITY = True
    mw = UserSessionsMiddleware(lambda request: None)
    request = rf.get("/")
    mw(request)
    assert UserSession.objects.count() == 0


@pytest.mark.parametrize("track_activity", [False, True])
def test_mw_with_request_user(rf, db, settings, user, track_activity):
    settings.USERSESSIONS_TRACK_ACTIVITY = track_activity
    mw = UserSessionsMiddleware(lambda request: None)
    request = rf.get("/")
    request.user = user
    request.session = Mock()
    request.session.session_key = "sess-123"
    mw(request)
    assert (
        UserSession.objects.filter(session_key="sess-123", user=user).exists()
        is track_activity
    )


def test_mw_with_anonymous_request_user(rf, db, settings):
    settings.USERSESSIONS_TRACK_ACTIVITY = True
    mw = UserSessionsMiddleware(lambda request: None)
    request = rf.get("/")
    request.user = AnonymousUser()
    request.session = Mock()
    request.session.session_key = "sess-123"
    mw(request)
    assert not UserSession.objects.exists()


@override_settings(USERSESSIONS_TRACK_ACTIVITY=True)
def test_mw_change_ip_and_useragent(rf, db, user):
    mw = UserSessionsMiddleware(lambda request: None)

    # First request
    request1 = rf.get("/")
    request1.user = user
    request1.session = Mock()
    request1.session.session_key = "sess-123"
    request1.META["HTTP_USER_AGENT"] = "Old User Agent"
    request1.META["REMOTE_ADDR"] = "1.1.1.1"
    mw(request1)

    # Second request with changed IP and User Agent
    request2 = rf.get("/")
    request2.user = user
    request2.session = Mock()
    request2.session.session_key = "sess-123"
    request2.META["HTTP_USER_AGENT"] = "New User Agent"
    request2.META["REMOTE_ADDR"] = "2.2.2.2"

    # Set up signal receiver
    signal_received = []

    def signal_handler(sender, request, from_session, to_session, **kwargs):
        signal_received.append((from_session, to_session))

    session_client_changed.connect(signal_handler)

    # Process second request
    mw(request2)

    # Check if UserSession was updated
    user_session = UserSession.objects.get(session_key="sess-123", user=user)
    assert user_session.ip == "2.2.2.2"
    assert user_session.user_agent == "New User Agent"

    # Check if signal was triggered
    assert len(signal_received) == 1
    from_session, to_session = signal_received[0]
    assert from_session.ip == "1.1.1.1"
    assert from_session.user_agent == "Old User Agent"
    assert to_session.ip == "2.2.2.2"
    assert to_session.user_agent == "New User Agent"

    # Clean up signal connection
    session_client_changed.disconnect(signal_handler)



import json
from http import HTTPStatus
from unittest.mock import patch

from django.urls import reverse

from pytest_django.asserts import assertTemplateUsed

from allauth.account.models import EmailAddress
from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.models import SocialAccount
from allauth.socialaccount.providers.base.constants import AuthProcess


def test_bad_redirect(client, headless_reverse, db, settings):
    settings.HEADLESS_ONLY = False
    resp = client.post(
        headless_reverse("headless:socialaccount:redirect_to_provider"),
        data={
            "provider": "dummy",
            "callback_url": "https://unsafe.org/hack",
            "process": AuthProcess.LOGIN,
        },
    )
    assertTemplateUsed(resp, "socialaccount/authentication_error.html")


def test_valid_redirect(client, headless_reverse, db):
    resp = client.post(
        headless_reverse("headless:socialaccount:redirect_to_provider"),
        data={
            "provider": "dummy",
            "callback_url": "/",
            "process": AuthProcess.LOGIN,
        },
    )
    assert resp.status_code == HTTPStatus.FOUND


def test_unknown_provider_redirect(client, headless_reverse, db, settings):
    resp = client.post(
        headless_reverse("headless:socialaccount:redirect_to_provider"),
        data={
            "provider": "unknown",
            "callback_url": "/",
            "process": AuthProcess.LOGIN,
        },
    )
    if getattr(settings, "HEADLESS_ONLY", False):
        assert resp.status_code == HTTPStatus.FOUND
    else:
        assert resp.status_code == HTTPStatus.UNAUTHORIZED


def test_manage_providers(auth_client, user, headless_reverse, provider_id):
    account_to_del = SocialAccount.objects.create(
        user=user, provider=provider_id, uid="p123"
    )
    account_to_keep = SocialAccount.objects.create(
        user=user, provider=provider_id, uid="p456"
    )
    resp = auth_client.get(
        headless_reverse("headless:socialaccount:manage_providers"),
    )
    data = resp.json()
    assert data["status"] == HTTPStatus.OK
    assert len(data["data"]) == 2
    resp = auth_client.delete(
        headless_reverse("headless:socialaccount:manage_providers"),
        data={"provider": account_to_del.provider, "account": account_to_del.uid},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.OK
    assert resp.json() == {
        "status": HTTPStatus.OK,
        "data": [
            {
                "display": "Unittest Server",
                "provider": {
                    "client_id": "Unittest client_id",
                    "openid_configuration_url": "https://unittest.example.com/.well-known/openid-configuration",
                    "flows": ["provider_redirect", "provider_token"],
                    "id": provider_id,
                    "name": "Unittest Server",
                },
                "uid": "p456",
            }
        ],
    }
    assert not SocialAccount.objects.filter(pk=account_to_del.pk).exists()
    assert SocialAccount.objects.filter(pk=account_to_keep.pk).exists()


def test_disconnect_bad_request(auth_client, user, headless_reverse, provider_id):
    resp = auth_client.delete(
        headless_reverse("headless:socialaccount:manage_providers"),
        data={"provider": provider_id, "account": "unknown"},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.BAD_REQUEST
    assert resp.json() == {
        "status": HTTPStatus.BAD_REQUEST,
        "errors": [{"code": "account_not_found", "message": "Unknown account."}],
    }


def test_disconnect_not_allowed(auth_client, user, headless_reverse, provider_id):
    user.set_unusable_password()
    user.save(update_fields=["password"])
    auth_client.force_login(user)
    account = SocialAccount.objects.create(user=user, uid="123", provider=provider_id)
    resp = auth_client.delete(
        headless_reverse("headless:socialaccount:manage_providers"),
        data={"provider": provider_id, "account": account.uid},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.BAD_REQUEST
    assert resp.json() == {
        "status": HTTPStatus.BAD_REQUEST,
        "errors": [
            {"code": "no_password", "message": "Your account has no password set up."}
        ],
    }


def test_valid_token(client, headless_reverse, db):
    id_token = json.dumps(
        {
            "id": 123,
            "email": "a@b.com",
            "email_verified": True,
        }
    )
    resp = client.post(
        headless_reverse("headless:socialaccount:provider_token"),
        data={
            "provider": "dummy",
            "token": {
                "id_token": id_token,
            },
            "process": AuthProcess.LOGIN,
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.OK
    assert EmailAddress.objects.filter(email="a@b.com", verified=True).exists()


def test_invalid_token(client, headless_reverse, db, google_provider_settings):
    resp = client.post(
        headless_reverse("headless:socialaccount:provider_token"),
        data={
            "provider": "google",
            "token": {
                "id_token": "dummy",
                "client_id": google_provider_settings["APPS"][0]["client_id"],
            },
            "process": AuthProcess.LOGIN,
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.BAD_REQUEST
    data = resp.json()
    assert data == {
        "status": HTTPStatus.BAD_REQUEST,
        "errors": [
            {"message": "Invalid token.", "code": "invalid_token", "param": "token"}
        ],
    }


def test_valid_token_multiple_apps(
    client, headless_reverse, db, google_provider_settings, settings, user_factory
):
    settings.ACCOUNT_EMAIL_VERIFICATION = "mandatory"
    google_provider_settings["APPS"].append(
        {"client_id": "client_id2", "secret": "secret2"}
    )
    id_token = {"sub": "uid-from-id-token", "email": "a@b.com", "email_verified": True}
    with patch(
        "allauth.socialaccount.providers.google.views._verify_and_decode",
        return_value=id_token,
    ):
        resp = client.post(
            headless_reverse("headless:socialaccount:provider_token"),
            data={
                "provider": "google",
                "token": {
                    "id_token": "dummy",
                    "client_id": google_provider_settings["APPS"][0]["client_id"],
                },
                "process": AuthProcess.LOGIN,
            },
            content_type="application/json",
        )
        assert resp.status_code == HTTPStatus.OK


def test_auth_error_no_headless_request(client, db, google_provider_settings, settings):
    """Authentication errors use the regular "Third-Party Login Failure"
    template if headless is not used.
    """
    settings.HEADLESS_ONLY = False
    resp = client.get(reverse("google_callback"))
    assertTemplateUsed(resp, "socialaccount/authentication_error.html")


def test_auth_error_headless_request(
    client, db, google_provider_settings, sociallogin_setup_state
):
    """Authentication errors redirect to the next URL with ?error params for
    headless requests.
    """
    state = sociallogin_setup_state(client, headless=True, next="/foo")
    resp = client.get(f"{reverse('google_callback')}?state={state}")
    assert resp["location"] == "/foo?error=unknown&error_process=login"


def test_auth_error_no_headless_state_request_headless_only(
    settings, client, db, google_provider_settings
):
    """Authentication errors redirect to a fallback error URL for headless-only,
    in case no next can be recovered from the state.
    """
    settings.HEADLESS_ONLY = True
    settings.HEADLESS_FRONTEND_URLS = {"socialaccount_login_error": "/3rdparty/failure"}
    resp = client.get(reverse("google_callback"))
    assert (
        resp["location"]
        == "http://testserver/3rdparty/failure?error=unknown&error_process=login"
    )


def test_auth_error_headless_state_request_headless_only(
    settings, client, db, google_provider_settings, sociallogin_setup_state
):
    """Authentication errors redirect to a fallback error URL for headless-only,
    in case no next can be recovered from the state.
    """
    state = sociallogin_setup_state(client, headless=True, next="/foo")
    settings.HEADLESS_ONLY = True
    settings.HEADLESS_FRONTEND_URLS = {"socialaccount_login_error": "/3rdparty/failure"}
    resp = client.get(f"{reverse('google_callback')}?state={state}")
    assert resp["location"] == "/foo?error=unknown&error_process=login"


def test_token_signup_closed(client, headless_reverse, db):
    id_token = json.dumps(
        {
            "id": 123,
            "email": "a@b.com",
            "email_verified": True,
        }
    )
    with patch(
        "allauth.socialaccount.adapter.DefaultSocialAccountAdapter.is_open_for_signup"
    ) as iofs:
        iofs.return_value = False
        resp = client.post(
            headless_reverse("headless:socialaccount:provider_token"),
            data={
                "provider": "dummy",
                "token": {
                    "id_token": id_token,
                },
                "process": AuthProcess.LOGIN,
            },
            content_type="application/json",
        )
        assert resp.status_code == HTTPStatus.FORBIDDEN
    assert not EmailAddress.objects.filter(email="a@b.com", verified=True).exists()


def test_provider_signup(client, headless_reverse, db, settings):
    settings.ACCOUNT_EMAIL_VERIFICATION = "mandatory"
    settings.ACCOUNT_EMAIL_REQUIRED = True
    settings.ACCOUNT_USERNAME_REQUIRED = False
    account_uid = "123"
    id_token = json.dumps(
        {
            "id": account_uid,
        }
    )
    resp = client.post(
        headless_reverse("headless:socialaccount:provider_token"),
        data={
            "provider": "dummy",
            "token": {
                "id_token": id_token,
            },
            "process": AuthProcess.LOGIN,
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    pending_flow = [f for f in resp.json()["data"]["flows"] if f.get("is_pending")][0]
    assert pending_flow["id"] == "provider_signup"

    resp = client.get(headless_reverse("headless:socialaccount:provider_signup"))
    assert resp.status_code == HTTPStatus.OK
    assert resp.json() == {
        "data": {
            "email": [],
            "account": {
                "display": "Dummy",
                "provider": {
                    "flows": ["provider_redirect", "provider_token"],
                    "id": "dummy",
                    "name": "Dummy",
                },
                "uid": account_uid,
            },
            "user": {"display": "user", "has_usable_password": False},
        },
        "status": HTTPStatus.OK,
    }

    resp = client.post(
        headless_reverse("headless:socialaccount:provider_signup"),
        data={
            "email": "a@b.com",
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    pending_flow = [f for f in resp.json()["data"]["flows"] if f.get("is_pending")][0]
    assert pending_flow["id"] == "verify_email"
    assert EmailAddress.objects.filter(email="a@b.com").exists()


def test_signup_closed(client, headless_reverse, db, settings):
    settings.ACCOUNT_EMAIL_VERIFICATION = "mandatory"
    settings.ACCOUNT_EMAIL_REQUIRED = True
    settings.ACCOUNT_USERNAME_REQUIRED = False
    id_token = json.dumps(
        {
            "id": 123,
        }
    )
    resp = client.post(
        headless_reverse("headless:socialaccount:provider_token"),
        data={
            "provider": "dummy",
            "token": {
                "id_token": id_token,
            },
            "process": AuthProcess.LOGIN,
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    pending_flow = [f for f in resp.json()["data"]["flows"] if f.get("is_pending")][0]
    assert pending_flow["id"] == "provider_signup"
    with patch(
        "allauth.socialaccount.adapter.DefaultSocialAccountAdapter.is_open_for_signup"
    ) as iofs:
        iofs.return_value = False
        resp = client.post(
            headless_reverse("headless:socialaccount:provider_signup"),
            data={
                "email": "a@b.com",
            },
            content_type="application/json",
        )
    assert resp.status_code == HTTPStatus.FORBIDDEN


def test_connect(user, auth_client, sociallogin_setup_state, headless_reverse, db):
    state = sociallogin_setup_state(
        auth_client, process="connect", next="/foo", headless=True
    )
    resp = auth_client.post(
        f"{reverse('dummy_authenticate')}?state={state}",
        data={
            "id": 123,
        },
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == "/foo"
    assert SocialAccount.objects.filter(user=user, provider="dummy", uid="123").exists()


def test_connect_reauthentication_required(
    user, auth_client, sociallogin_setup_state, headless_reverse, db, settings
):
    settings.ACCOUNT_REAUTHENTICATION_REQUIRED = True

    state = sociallogin_setup_state(
        auth_client, process="connect", next="/foo", headless=True
    )
    resp = auth_client.post(
        f"{reverse('dummy_authenticate')}?state={state}",
        data={
            "id": 123,
        },
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert (
        resp["location"] == "/foo?error=reauthentication_required&error_process=connect"
    )


def test_connect_already_connected(
    user, user_factory, auth_client, sociallogin_setup_state, headless_reverse, db
):
    # The other user already connected the account.
    other_user = user_factory()
    SocialAccount.objects.create(user=other_user, uid="123", provider="dummy")
    # Then, this user tries to connect...
    state = sociallogin_setup_state(
        auth_client, process=AuthProcess.CONNECT, next="/foo", headless=True
    )
    resp = auth_client.post(
        f"{reverse('dummy_authenticate')}?state={state}",
        data={
            "id": 123,
        },
    )
    # We're redirected, and an error code is shown.
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == "/foo?error=connected_other&error_process=connect"
    assert not SocialAccount.objects.filter(
        user=user, provider="dummy", uid="123"
    ).exists()


def test_token_connect(user, auth_client, headless_reverse, db):
    id_token = json.dumps(
        {
            "id": 123,
            "email": "a@b.com",
            "email_verified": True,
        }
    )
    resp = auth_client.post(
        headless_reverse("headless:socialaccount:provider_token"),
        data={
            "provider": "dummy",
            "token": {
                "id_token": id_token,
            },
            "process": AuthProcess.CONNECT,
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.OK
    assert SocialAccount.objects.filter(uid="123", user=user).exists()


def test_token_connect_already_connected(
    user, auth_client, headless_reverse, db, user_factory
):
    # The other user already connected the account.
    other_user = user_factory()
    SocialAccount.objects.create(user=other_user, uid="123", provider="dummy")
    id_token = json.dumps(
        {
            "id": 123,
            "email": "a@b.com",
            "email_verified": True,
        }
    )
    resp = auth_client.post(
        headless_reverse("headless:socialaccount:provider_token"),
        data={
            "provider": "dummy",
            "token": {
                "id_token": id_token,
            },
            "process": AuthProcess.CONNECT,
        },
        content_type="application/json",
    )
    assert not SocialAccount.objects.filter(uid="123", user=user).exists()
    assert resp.status_code == HTTPStatus.BAD_REQUEST
    assert resp.json() == {
        "status": HTTPStatus.BAD_REQUEST,
        "errors": [
            {
                "code": "connected_other",
                "message": "The third-party account is already connected to a different account.",
            }
        ],
    }


def test_provider_signup_not_pending(client, headless_reverse, db, settings):
    resp = client.post(
        headless_reverse("headless:socialaccount:provider_signup"),
        data={
            "email": "a@b.com",
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.CONFLICT


def test_valid_openid_connect_token(
    client, headless_reverse, db, openid_connect_provider_id
):
    provider = get_adapter().get_provider(None, openid_connect_provider_id)
    id_token = json.dumps(
        {
            "id": 123,
            "email": "a@b.com",
            "email_verified": True,
        }
    )
    with patch(
        "allauth.socialaccount.providers.openid_connect.views.OpenIDConnectOAuth2Adapter.openid_config"
    ):
        with patch("allauth.socialaccount.internal.jwtkit.verify_and_decode") as vad:
            vad.return_value = {"sub": "123"}
            resp = client.post(
                headless_reverse("headless:socialaccount:provider_token"),
                data={
                    "provider": openid_connect_provider_id,
                    "token": {
                        "client_id": provider.app.client_id,
                        "id_token": id_token,
                    },
                    "process": AuthProcess.LOGIN,
                },
                content_type="application/json",
            )
            assert resp.status_code == HTTPStatus.OK
            assert SocialAccount.objects.filter(uid="123").exists()



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.zoho.provider import ZohoProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class ZohoTests(OAuth2TestsMixin, TestCase):
    provider_id = ZohoProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """
{"First_Name":"John","Email":"jdoe@example.com",
"Last_Name":"Doe","Display_Name":"JDoee","ZUID":1234567}
""",
        )

    def get_expected_to_str(self):
        return "jdoe@example.com"



import typing

from django.contrib import messages

from example.users.models import User

from allauth.account.adapter import DefaultAccountAdapter


class AccountAdapter(DefaultAccountAdapter):

    def set_phone(self, user, phone: str, verified: bool):
        user.phone = phone
        user.phone_verified = verified
        user.save(update_fields=["phone", "phone_verified"])

    def get_phone(self, user) -> typing.Optional[typing.Tuple[str, bool]]:
        if user.phone:
            return user.phone, user.phone_verified
        return None

    def set_phone_verified(self, user, phone):
        self.set_phone(user, phone, True)

    def send_verification_code_sms(self, user, phone: str, code: str, **kwargs):
        messages.add_message(
            self.request,
            messages.WARNING,
            f"⚠️ SMS demo stub: assume code {code} was sent to {phone}.",
        )

    def send_unknown_account_sms(self, phone: str, **kwargs):
        messages.add_message(
            self.request,
            messages.WARNING,
            f"⚠️ SMS demo stub: Enumeration prevention: texted {phone} informing no account exists.",
        )

    def send_account_already_exists_sms(self, phone: str, **kwargs):
        messages.add_message(
            self.request,
            messages.WARNING,
            f"⚠️ SMS demo stub: Enumeration prevention: texted {phone} informing account already exists.",
        )

    def get_user_by_phone(self, phone):
        return User.objects.filter(phone=phone).order_by("-phone_verified").first()



import uuid

from django.contrib.auth.models import AbstractUser
from django.db import models


class User(AbstractUser):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    phone = models.CharField(max_length=16, unique=True, blank=True, null=True)
    phone_verified = models.BooleanField(default=False)



from http import HTTPStatus

from django.urls import reverse


def test_case_insensitive_password_reset(settings, enable_cache, user_factory, client):
    settings.ACCOUNT_RATE_LIMITS = {"reset_password": "1/m"}
    user_factory(email="a@b.com")
    resp = client.post(reverse("account_reset_password"), data={"email": "a@b.com"})
    assert resp.status_code == HTTPStatus.FOUND
    resp = client.post(reverse("account_reset_password"), data={"email": "A@B.COM"})
    assert resp.status_code == HTTPStatus.TOO_MANY_REQUESTS



from http import HTTPStatus
from unittest.mock import ANY

from django.urls import reverse

import pytest
from pytest_django.asserts import assertTemplateUsed

from allauth.account.authentication import AUTHENTICATION_METHODS_SESSION_KEY
from allauth.mfa.models import Authenticator


def test_reauthentication(auth_client, user_with_recovery_codes):
    resp = auth_client.get(reverse("mfa_view_recovery_codes"))
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"].startswith(reverse("account_reauthenticate"))
    resp = auth_client.get(reverse("mfa_reauthenticate"))
    assertTemplateUsed(resp, "mfa/reauthenticate.html")
    authenticator = Authenticator.objects.get(
        user=user_with_recovery_codes, type=Authenticator.Type.RECOVERY_CODES
    )
    unused_code = authenticator.wrap().get_unused_codes()[0]
    resp = auth_client.post(reverse("mfa_reauthenticate"), data={"code": unused_code})
    assert resp.status_code == HTTPStatus.FOUND
    resp = auth_client.get(reverse("mfa_view_recovery_codes"))
    assert resp.status_code == HTTPStatus.OK
    assertTemplateUsed(resp, "mfa/recovery_codes/index.html")
    methods = auth_client.session[AUTHENTICATION_METHODS_SESSION_KEY]
    assert methods[-1] == {
        "method": "mfa",
        "type": "recovery_codes",
        "id": authenticator.pk,
        "at": ANY,
        "reauthenticated": True,
    }


@pytest.mark.parametrize(
    "url_name",
    (
        "mfa_activate_totp",
        "mfa_index",
        "mfa_deactivate_totp",
    ),
)
def test_login_required_views(client, url_name):
    resp = client.get(reverse(url_name))
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"].startswith(reverse("account_login"))


def test_index(auth_client, user_with_totp):
    resp = auth_client.get(reverse("mfa_index"))
    assert "authenticators" in resp.context


@pytest.mark.parametrize("allowed", [False, True])
def test_add_email_not_allowed(
    auth_client,
    user_with_totp,
    settings,
    allowed,
):
    settings.MFA_ALLOW_UNVERIFIED_EMAIL = allowed
    resp = auth_client.post(
        reverse("account_email"),
        {"action_add": "", "email": "change-to@this.org"},
    )
    if allowed:
        assert resp.status_code == HTTPStatus.FOUND
    else:
        assert resp.status_code == HTTPStatus.OK
        assert resp.context["form"].errors == {
            "email": [
                "You cannot add an email address to an account protected by two-factor authentication."
            ]
        }


def test_add_email_allowed_when_verification_by_code(
    auth_client,
    user_with_totp,
    settings,
):
    settings.ACCOUNT_EMAIL_VERIFICATION_BY_CODE_ENABLED = True
    resp = auth_client.post(
        reverse("account_email"),
        {"action_add": "", "email": "change-to@this.org"},
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("account_email_verification_sent")



import time
import uuid
from datetime import timedelta
from types import SimpleNamespace

from django.utils import timezone

import pytest

from allauth.core.context import request_context
from allauth.idp.oidc.adapter import get_adapter
from allauth.idp.oidc.internal.oauthlib.request_validator import (
    OAuthLibRequestValidator,
)
from allauth.idp.oidc.models import Client, Token


@pytest.fixture
def oidc_client_secret():
    return uuid.uuid4().hex


@pytest.fixture
def oidc_client(db, oidc_client_secret):
    client = Client.objects.create()
    client.set_secret(oidc_client_secret)

    client.set_redirect_uris(["https://client/callback"])
    client.set_scopes(["profile", "openid", "email"])
    client.set_grant_types([g.value for g in Client.GrantType])
    client.set_response_types(["code", "token"])
    client.save()
    return client


@pytest.fixture
def device_client(db):
    client = Client.objects.create()
    client.type = Client.Type.PUBLIC
    client.set_redirect_uris(["https://client/callback"])
    client.set_scopes(["profile", "openid", "email"])
    client.set_grant_types([g.value for g in Client.GrantType])
    client.set_response_types(["code", "token"])
    client.save()
    return client


@pytest.fixture
def id_token_generator(rf):
    def f(client, user):
        with request_context(rf.get("/")):
            request = SimpleNamespace(client=client, user=user, scopes=["openid"])
            return OAuthLibRequestValidator().finalize_id_token(
                {
                    "aud": client.id,
                    "iat": int(time.time()),
                },
                {},
                None,
                request,
            )

    return f


@pytest.fixture
def access_token_generator():
    def f(client, user, scopes=["openid"]):
        token = uuid.uuid4().hex
        token_hash = get_adapter().hash_token(token)
        instance = Token(
            type=Token.Type.ACCESS_TOKEN,
            user=user,
            client=client,
            hash=token_hash,
        )
        instance.set_scopes(scopes)
        instance.save()
        return token, instance

    return f


@pytest.fixture
def refresh_token_factory():
    def f(*, user, client, scopes=None):
        adapter = get_adapter()
        value = uuid.uuid4().hex
        rt = Token.objects.create(
            client=client,
            user=user,
            type=Token.Type.REFRESH_TOKEN,
            hash=adapter.hash_token(value),
            expires_at=timezone.now() + timedelta(seconds=60),
        )
        if scopes is None:
            scopes = ["openid", "profile"]
        rt.set_scopes(scopes)
        rt.save()
        return value, rt

    return f



from allauth.account.forms import ResetPasswordForm


def test_user_email_unicode_collision(settings, rf, user_factory, mailoutbox):
    settings.ACCOUNT_PREVENT_ENUMERATION = False
    user_factory(username="mike123", email="mike@example.org")
    user_factory(username="mike456", email="mıke@example.org")
    data = {"email": "mıke@example.org"}
    form = ResetPasswordForm(data)
    assert form.is_valid()
    form.save(rf.get("/"))
    assert len(mailoutbox) == 1
    assert mailoutbox[0].to == ["mıke@example.org"]


def test_user_email_domain_unicode_collision(settings, rf, user_factory, mailoutbox):
    settings.ACCOUNT_PREVENT_ENUMERATION = False
    user_factory(username="mike123", email="mike@ixample.org")
    user_factory(username="mike456", email="mike@ıxample.org")
    data = {"email": "mike@ıxample.org"}
    form = ResetPasswordForm(data)
    assert form.is_valid()
    form.save(rf.get("/"))
    assert len(mailoutbox) == 1
    assert mailoutbox[0].to == ["mike@ıxample.org"]


def test_user_email_unicode_collision_nonexistent(settings, user_factory):
    settings.ACCOUNT_PREVENT_ENUMERATION = False
    user_factory(username="mike123", email="mike@example.org")
    data = {"email": "mıke@example.org"}
    form = ResetPasswordForm(data)
    assert not form.is_valid()


def test_user_email_domain_unicode_collision_nonexistent(settings, user_factory):
    settings.ACCOUNT_PREVENT_ENUMERATION = False
    user_factory(username="mike123", email="mike@ixample.org")
    data = {"email": "mike@ıxample.org"}
    form = ResetPasswordForm(data)
    assert not form.is_valid()



from http import HTTPStatus
from unittest.mock import ANY

from django.urls import reverse

import pytest

from allauth.idp.oidc.models import Token


def test_userinfo_bad_token(client, oidc_client, user):
    # Pass along ID token as hint
    resp = client.get(reverse("idp:oidc:userinfo"))
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    assert resp.json() == {
        "error": "invalid_token",
        "error_description": "The access token provided is expired, revoked, malformed, or invalid for other reasons.",
    }


def test_revoke_access_token(
    client, oidc_client, oidc_client_secret, user, access_token_generator
):
    token, instance = access_token_generator(oidc_client, user)
    _, instance_to_keep = access_token_generator(oidc_client, user)
    resp = client.post(
        reverse("idp:oidc:revoke"),
        data={
            "client_id": oidc_client.id,
            "client_secret": oidc_client_secret,
            "token": token,
        },
    )
    assert resp.status_code == HTTPStatus.OK
    assert not Token.objects.filter(pk=instance.pk).exists()
    assert Token.objects.filter(pk=instance_to_keep.pk).exists()


@pytest.mark.parametrize("rotate_refresh_token", [False, True])
@pytest.mark.parametrize("scopes", [("openid",), ("openid", "email")])
def test_refresh_token(
    db,
    client,
    oidc_client,
    oidc_client_secret,
    user,
    refresh_token_factory,
    settings,
    rotate_refresh_token,
    scopes,
):
    settings.IDP_OIDC_ROTATE_REFRESH_TOKEN = rotate_refresh_token
    rt, rt_instance = refresh_token_factory(
        user=user, client=oidc_client, scopes=scopes
    )
    rt_instance.set_scope_email("a@b.org")
    rt_instance.save()
    resp = client.post(
        reverse("idp:oidc:token"),
        {
            "refresh_token": rt,
            "grant_type": "refresh_token",
            "client_id": oidc_client.id,
            "client_secret": oidc_client_secret,
        },
    )
    assert resp.status_code == HTTPStatus.OK
    data = resp.json()
    new_rt_instance = Token.objects.lookup(
        Token.Type.REFRESH_TOKEN, data["refresh_token"]
    )
    if rotate_refresh_token:
        assert data["refresh_token"] != rt
        assert new_rt_instance.pk != rt_instance.pk
    else:
        assert data["refresh_token"] == rt
    assert Token.objects.filter(type=Token.Type.REFRESH_TOKEN).count() == 1
    token = Token.objects.lookup(Token.Type.ACCESS_TOKEN, data["access_token"])

    assert token.get_scope_email() == ("a@b.org" if "email" in scopes else None)
    return
    assert data == {
        "access_token": ANY,
        "expires_in": 3600,
        "refresh_token": ANY,
        "scope": "openid profile",
        "token_type": "Bearer",
    }


def test_revoke_refresh_token(
    db, client, oidc_client, oidc_client_secret, user, refresh_token_factory
):
    token_value, token_instance = refresh_token_factory(user=user, client=oidc_client)
    resp = client.post(
        reverse("idp:oidc:revoke"),
        {
            "token": token_value,
            "client_id": oidc_client.id,
            "client_secret": oidc_client_secret,
        },
    )
    assert resp.status_code == HTTPStatus.OK
    assert resp.content == b""
    assert not Token.objects.filter(pk=token_instance.pk).exists()



import pytest

from allauth.core.internal.cryptokit import compare_user_code, generate_user_code


@pytest.mark.parametrize(
    "actual,expected,result",
    [
        ("xkcd", "XKCD", True),
        ("XKCD", "xkcd", True),
        ("XK-CD", "XKCD", True),
        ("XK CD", "XKCD", True),
        ("XK - CD", "XKCD", True),
        ("XKCD", "XK-CD", True),
        ("ABCD", "XKCD", False),
        ("", "XKCD", False),
        ("XKCD", "", False),
        ("", "", False),
    ],
)
def test_compare_user_code(actual, expected, result):
    assert compare_user_code(actual=actual, expected=expected) is result


@pytest.mark.parametrize(
    "length,result",
    [
        (2, "X-X"),
        (3, "XX-X"),
        (6, "XXX-XXX"),
        (8, "XXXX-XXXX"),
        (9, "XXX-XXX-XXX"),
        (15, "XXXXX-XXXXX-XXXXX"),
    ],
)
def test_generate_user_code_dashing(length, result):
    assert generate_user_code(length=length, allowed_chars="X", dashed=True) == result



from datetime import date, datetime

from django.contrib.postgres.fields import ArrayField
from django.core.files.base import ContentFile
from django.db import models

from allauth.core.internal import modelkit


def test_serializer():
    class SomeValue:
        pass

    some_value = SomeValue()

    class SomeField(models.Field):
        def get_prep_value(self, value):
            return "somevalue"

        def from_db_value(self, value, expression, connection):
            return some_value

    class SomeModel(models.Model):
        dt = models.DateTimeField()
        t = models.TimeField()
        d = models.DateField()
        img1 = models.ImageField()
        img2 = models.ImageField()
        img3 = models.ImageField()
        something = SomeField()
        ips = ArrayField(models.GenericIPAddressField(), default=list, blank=True)

        class Meta:
            app_label = "dummy"

    def method(self):
        pass

    instance = SomeModel(
        dt=datetime.now(),
        d=date.today(),
        something=some_value,
        t=datetime.now().time(),
        ips=["1.1.1.1", "1.2.3.4"],
    )
    instance.img1 = ContentFile(b"%PDF", name="foo.pdf")
    instance.img2 = ContentFile(
        b"\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x01\x00\x00\x00\x01\x01\x00"
        b"\x00\x00\x007n\xf9$\x00\x00\x00\nIDATx\x9cc`\x00\x00\x00\x02\x00\x01H\xaf"
        b"\xa4q\x00\x00\x00\x00IEND\xaeB`\x82",
        name="foo.png",
    )
    # make sure serializer doesn't fail if a method is attached to
    # the instance
    instance.method = method
    instance.nonfield = "hello"
    data = modelkit.serialize_instance(instance)
    instance2 = modelkit.deserialize_instance(SomeModel, data)
    assert getattr(instance, "method", None) == method
    assert getattr(instance2, "method", None) is None
    assert instance2.something == some_value
    assert instance2.img1.name == "foo.pdf"
    assert instance2.img2.name == "foo.png"
    assert instance2.img3.name == ""
    assert instance.nonfield == instance2.nonfield
    assert instance.d == instance2.d
    assert instance.dt.date() == instance2.dt.date()
    assert instance.ips == instance2.ips
    for t1, t2 in [
        (instance.t, instance2.t),
        (instance.dt.time(), instance2.dt.time()),
    ]:
        assert t1.hour == t2.hour
        assert t1.minute == t2.minute
        assert t1.second == t2.second
        # AssertionError: datetime.time(10, 6, 28, 705776)
        #     != datetime.time(10, 6, 28, 705000)
        assert int(t1.microsecond / 1000) == int(t2.microsecond / 1000)


def test_serializer_binary_field():
    class SomeBinaryModel(models.Model):
        bb = models.BinaryField()
        bb_empty = models.BinaryField()

        class Meta:
            app_label = "dummy"

    instance = SomeBinaryModel(bb=b"some binary data")

    serialized = modelkit.serialize_instance(instance)
    deserialized = modelkit.deserialize_instance(SomeBinaryModel, serialized)

    assert serialized["bb"] == "c29tZSBiaW5hcnkgZGF0YQ=="
    assert serialized["bb_empty"] == ""
    assert deserialized.bb == b"some binary data"
    assert deserialized.bb_empty == b""



from django.contrib.auth.middleware import LoginRequiredMiddleware


class CustomLoginRequiredMiddleware(LoginRequiredMiddleware):
    def handle_no_permission(self, request, view_func):
        # Quick & dirty workaround for: https://github.com/vitalik/django-ninja/issues/1461
        if "/ninja/" in request.path:
            return None
        return super().handle_no_permission(request, view_func)



from django.urls import path

from drf_spectacular.views import (
    SpectacularAPIView,
    SpectacularRedocView,
    SpectacularSwaggerView,
)

from .views import AddAPIView


urlpatterns = [
    path("api/add/", AddAPIView.as_view()),
    path("api/schema/", SpectacularAPIView.as_view(), name="schema"),
    path(
        "api/schema/swagger-ui/",
        SpectacularSwaggerView.as_view(url_name="schema"),
        name="swagger-ui",
    ),
    path(
        "api/schema/redoc/",
        SpectacularRedocView.as_view(url_name="schema"),
        name="redoc",
    ),
]



from http import HTTPStatus

from django.contrib.auth.models import User
from django.test import TestCase

from allauth.socialaccount.models import SocialAccount
from allauth.socialaccount.providers.strava.provider import StravaProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class StravaTests(OAuth2TestsMixin, TestCase):
    provider_id = StravaProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """{
                "id": 32641234,
                "username": null,
                "resource_state": 2,
                "firstname": "georges",
                "lastname": "camembert",
                "city": "London",
                "state": "England",
                "country": "United Kingdom",
                "sex": "M",
                "premium": false,
                "summit": false,
                "created_at": "2017-07-12T12:42:52Z",
                "updated_at": "2017-10-21T11:01:23Z",
                "badge_type_id": 0,
                "profile_medium": "avatar/athlete/medium.png",
                "profile": "avatar/athlete/large.png",
                "friend": null,
                "follower": null,
                "email": "bill@example.com"
            }""",
        )  # noqa

    def get_expected_to_str(self):
        return "bill@example.com"

    def get_mocked_response_avatar_invalid_id(self):
        """Profile including realistic avatar URL
        user ID set to 0 to test edge case where id would be missing"""
        return MockedResponse(
            HTTPStatus.OK,
            """{
                "id": 0,
                "username": null,
                "resource_state": 2,
                "firstname": "georges",
                "lastname": "camembert",
                "city": "London",
                "state": "England",
                "country": "United Kingdom",
                "sex": "M",
                "premium": false,
                "summit": false,
                "created_at": "2017-07-12T12:42:52Z",
                "updated_at": "2017-10-21T11:01:23Z",
                "badge_type_id": 0,
                "profile_medium": "https://cloudfront.net/1/medium.jpg",
                "profile": "https://cloudfront.net/1/large.jpg",
                "friend": null,
                "follower": null,
                "email": "bill@example.com"
            }""",
        )  # noqa

    def test_valid_avatar(self):
        """test response with Avatar URL"""
        self.login(self.get_mocked_response_avatar_invalid_id())
        user = User.objects.get(email="bill@example.com")
        soc_acc = SocialAccount.objects.filter(
            user=user, provider=self.provider.id
        ).get()
        provider_account = soc_acc.get_provider_account()
        self.assertEqual(
            provider_account.get_avatar_url(),
            "https://cloudfront.net/1/large.jpg",
        )
        self.assertIsNone(provider_account.get_profile_url())

    def get_login_response_json(self, with_refresh_token=True):
        rt = ""
        if with_refresh_token:
            rt = ',"refresh_token": "testrf"'
        return (
            """{
            "uid":"weibo",
            "access_token":"testac",
            "livemode": false,
            "token_type": "bearer",
            "strava_publishable_key": "pk_test_someteskey",
            "strava_user_id": "acct_sometestid",
            "scope": "read_write"
            %s }"""
            % rt
        )



from http import HTTPStatus
from unittest.mock import ANY

from django.core.cache import cache
from django.urls import reverse
from django.utils.http import urlencode

import pytest
from pytest_django.asserts import assertTemplateUsed

from allauth.idp.oidc import app_settings
from allauth.idp.oidc.internal.oauthlib import device_codes
from allauth.idp.oidc.models import Client, Token


@pytest.fixture
def poll_sleep():
    def f(device_code, seconds):
        data = cache.get(device_codes.cache_device_code_key(device_code))
        if data:
            data["last_poll_at"] -= seconds
            device_codes.update_device_state(device_code, data)

    return f


def test_device_flow_invalid_client(
    db,
    client,
):
    payload = {
        "client_id": "unknown",
    }
    resp = client.post(
        reverse("idp:oidc:device_code"),
        data=urlencode(payload),
        content_type="application/x-www-form-urlencoded",
    )
    assert resp.status_code == HTTPStatus.BAD_REQUEST
    assert resp.json() == {
        "error": "invalid_request",
        "error_description": "Invalid client_id parameter value.",
    }


def test_device_flow_invalid_scope(db, client, device_client):
    payload = {
        "client_id": device_client.id,
        "scope": "openid wrong",
    }
    resp = client.post(
        reverse("idp:oidc:device_code"),
        data=urlencode(payload),
        content_type="application/x-www-form-urlencoded",
    )
    assert resp.status_code == HTTPStatus.BAD_REQUEST
    assert resp.json() == {"error": "invalid_scope"}


@pytest.mark.parametrize("action", ["deny", "confirm"])
@pytest.mark.parametrize("with_scope", [False, True])
def test_device_flow(
    client,
    auth_client,
    device_client,
    enable_cache,
    action,
    user,
    with_scope,
    poll_sleep,
):
    device_client.set_default_scopes(["profile"])
    device_client.save()
    payload = {
        "client_id": device_client.id,
    }
    if with_scope:
        payload["scope"] = "openid email"
    resp = client.post(
        reverse("idp:oidc:device_code"),
        data=urlencode(payload),
        content_type="application/x-www-form-urlencoded",
    )
    assert resp.status_code == HTTPStatus.OK
    data = resp.json()
    assert data == {
        "verification_uri": "http://testserver/identity/o/device",
        "verification_uri_complete": ANY,
        "expires_in": app_settings.DEVICE_CODE_EXPIRES_IN,
        "user_code": ANY,
        "device_code": ANY,
        "interval": 5,
    }
    assert (
        data["verification_uri_complete"]
        == f"{data['verification_uri']}?code={data['user_code']}"
    )

    resp = client.post(
        reverse("idp:oidc:token"),
        {
            "device_code": data["device_code"],
            "grant_type": Client.GrantType.DEVICE_CODE,
            "client_id": device_client.id,
        },
    )
    assert resp.status_code == HTTPStatus.BAD_REQUEST
    assert resp.json() == {
        "error": "authorization_pending",
    }

    resp = auth_client.get(data["verification_uri"])
    assert resp.status_code == HTTPStatus.OK
    assertTemplateUsed(resp, "idp/oidc/device_authorization_code_form.html")
    assert resp.context["form"].errors == {}

    resp = auth_client.get(f"{data['verification_uri']}?code=wrong")
    assert resp.status_code == HTTPStatus.OK
    assertTemplateUsed(resp, "idp/oidc/device_authorization_code_form.html")
    assert resp.context["form"].errors == {"code": ["Incorrect code."]}

    resp = auth_client.get(data["verification_uri_complete"])
    assert resp.status_code == HTTPStatus.OK
    assertTemplateUsed(resp, "idp/oidc/device_authorization_confirm_form.html")

    confirmed = action == "confirm"
    resp = auth_client.post(data["verification_uri_complete"], {"action": action})
    assert resp.status_code == HTTPStatus.OK
    assertTemplateUsed(
        resp,
        (
            "idp/oidc/device_authorization_confirmed.html"
            if confirmed
            else "idp/oidc/device_authorization_denied.html"
        ),
    )

    poll_sleep(data["device_code"], 10)
    resp = client.post(
        reverse("idp:oidc:token"),
        {
            "device_code": data["device_code"],
            "grant_type": Client.GrantType.DEVICE_CODE,
            "client_id": device_client.id,
        },
    )
    if confirmed:
        assert resp.status_code == HTTPStatus.OK
        assert resp.json() == {
            "access_token": ANY,
            "expires_in": 3600,
            "refresh_token": ANY,
            "scope": "openid email" if with_scope else "profile",
            "token_type": "Bearer",
        }
        token = Token.objects.lookup(
            Token.Type.ACCESS_TOKEN, resp.json()["access_token"]
        )
        assert token.user_id == user.pk
        # Single-use device codes.
        poll_sleep(data["device_code"], 10)
        resp = client.post(
            reverse("idp:oidc:token"),
            {
                "device_code": data["device_code"],
                "grant_type": Client.GrantType.DEVICE_CODE,
                "client_id": device_client.id,
            },
        )
        assert resp.status_code == HTTPStatus.BAD_REQUEST
        assert resp.json() == {
            "error": "invalid_grant",
        }

    else:
        assert resp.status_code == HTTPStatus.BAD_REQUEST
        assert resp.json() == {
            "error": "access_denied",
        }


def test_slow_down_flow(client, device_client, enable_cache, poll_sleep):
    payload = {
        "client_id": device_client.id,
    }
    resp = client.post(
        reverse("idp:oidc:device_code"),
        data=urlencode(payload),
        content_type="application/x-www-form-urlencoded",
    )
    assert resp.status_code == HTTPStatus.OK
    data = resp.json()

    for sleep, error in [
        (0, "authorization_pending"),
        (3, "slow_down"),
        (3, "authorization_pending"),
    ]:
        if sleep:
            poll_sleep(data["device_code"], sleep)
        resp = client.post(
            reverse("idp:oidc:token"),
            {
                "device_code": data["device_code"],
                "grant_type": Client.GrantType.DEVICE_CODE,
                "client_id": device_client.id,
            },
        )
        assert resp.status_code == HTTPStatus.BAD_REQUEST
        assert resp.json() == {"error": error}
